import os
import subprocess

from afeng_tools.log_tool.loguru_tools import get_logger, log_error

logger = get_logger()


def download_file(down_url:str, save_path:str, save_file_name:str=None, referer:str=''):
    try:
        subprocess.run(['aria2c', '-h'], capture_output=True)
        logger.info('发现 aria2c, 将使用 aria2c 下载文件')
        cmd = ' '.join([
            f'aria2c "{down_url}"',
            f'--referer={referer}',
            f'-d "{save_path}"',
            f'-o "{save_file_name}"'
            f'-x 16',
            f'-s 8',
        ])
        # print(cmd)
        os.system(cmd)

    except FileNotFoundError as e:
        log_error(logger, '未发现 aria2c', e)



"""
- pip install pydantic -i https://pypi.tuna.tsinghua.edu.cn/simple/
"""
from datetime import datetime
from typing import Optional, List

from pydantic import BaseModel


class TokenInfo(BaseModel):
    """Token信息"""
    # 获取到的Access Token
    access_token: str
    # Access Token的有效期，单位为秒。
    expires_in: int
    # 用于刷新 Access Token, 有效期为10年。
    refresh_token: str
    # Access Token 最终的访问权限，即用户的实际授权列表。
    scope: str
    # token获取的时间，用于判断token是否过期
    token_time: datetime


class GrantTokenInfo(BaseModel):
    """简化模式 Token信息"""
    # 获取到的Access Token
    access_token: str
    # Access Token的有效期，单位为秒。
    expires_in: int
    session_secret: str
    session_key: str
    # Access Token 最终的访问权限，即用户的实际授权列表。
    scope: str
    # token获取的时间，用于判断token是否过期
    token_time: datetime


class DeviceCodeInfo(BaseModel):
    """设备码信息"""
    # 设备码，可用于生成单次凭证 Access Token。
    device_code: str
    # 用户码。如果选择让用户输入 user code 方式，来引导用户授权，设备需要展示 user code 给用户。
    user_code: str
    # 用户输入 user code 进行授权的 url。
    verification_url: str
    # 二维码url，用户用手机等智能终端扫描该二维码完成授权。
    qrcode_url: str
    # device_code 的过期时间，单位：秒。到期后 device_code 不能换 Access Token。
    expires_in: int
    # device_code 换 Access Token 轮询间隔时间，单位：秒。 轮询次数限制小于 expire_in/interval。
    interval: int


class PanVolumeInfo(BaseModel):
    """百度网盘容量信息"""
    # 总空间大小，单位B
    total: int
    # 7天内是否有容量到期
    expire: Optional[bool] = None
    # 已使用大小，单位B
    used: int
    # 免费容量，单位B
    free: int


class UserInfo(BaseModel):
    """用户的基本信息"""
    # 百度账号
    baidu_name: str
    # 网盘账号
    net_disk_name: str
    # 头像地址
    avatar_url: str
    # 会员类型，0普通用户、1普通会员、2超级会员
    vip_type: int
    # 用户ID
    uk: int


class FileThumb(BaseModel):
    """文件图片缩略图信息"""
    # 140x90大小的图片路径
    url1: Optional[str] = None
    # 360x270大小的图片路径
    url2: Optional[str] = None
    # 850x580大小的图片路径
    url3: Optional[str] = None


class FileInfo(BaseModel):
    """文件信息"""
    # 文件在云端的唯一标识ID
    fs_id: int
    # 文件的绝对路径
    path: str
    # 文件名称
    server_filename: str
    # 文件大小，单位B
    size: int
    # 文件在服务器创建时间
    server_ctime: Optional[int] = None
    # 文件在服务器修改时间
    server_mtime: Optional[int] = None
    # 文件在客户端修改时间
    local_mtime: Optional[int] = None
    # 文件在客户端创建时间
    local_ctime: Optional[int] = None
    # 是否为目录，0 文件、1 目录
    isdir: Optional[int] = None
    # 文件类型，1 视频、2 音频、3 图片、4 文档、5 应用、6 其他、7 种子
    category: Optional[int] = None
    # 如：pdf
    real_category: Optional[str] = None
    # 云端哈希（非文件真实MD5），只有是文件类型时，该字段才存在
    md5: Optional[str] = None
    # 该目录是否存在子目录，只有请求参数web=1且该条目为目录时，该字段才存在， 0为存在， 1为不存在
    dir_empty: Optional[int] = None
    # 只有请求参数web=1且该条目分类为图片时，该字段才存在，包含三个尺寸的缩略图URL
    thumbs: Optional[FileThumb] = None
    server_atime: Optional[int] = None
    tkbind_id: Optional[int] = None
    unlist: Optional[int] = None
    wpfile: Optional[int] = None
    share: Optional[int] = None
    pl: Optional[int] = None
    docpreview: Optional[str] = None
    owner_id: Optional[int] = None
    owner_type: Optional[int] = None
    lodocpreview: Optional[str] = None
    extent_tinyint7: Optional[int] = None
    from_type: Optional[int] = None
    oper_id: Optional[int] = None


class DocFileInfo(BaseModel):
    """文档文件信息"""
    # 文件在云端的唯一标识ID
    fs_id: int
    # 文件名称
    server_filename: str
    # 文件的绝对路径
    path: str
    # 文件类型，1 视频、2 音频、3 图片、4 文档、5 应用、6 其他、7 种子
    category: int
    # 是否为目录，0 文件、1 目录
    isdir: int
    # 文件大小，单位B
    size: int
    # 图片缩略图，包含三个尺寸
    thumbs: Optional[FileThumb] = None
    # 文档预览地址 （已下线）
    lodocpreview: Optional[str] = None
    # 用户获取文档预览地址。如果接口只下发了docpreview，那么文档预览地址=docpreview + "type=pdf&from=lo"。（已下线）
    docpreview: Optional[str] = None
    # 文件在客户端创建时间
    local_ctime: Optional[int] = None
    # 文件在客户端修改时间
    local_mtime: Optional[int] = None
    # 文件在服务端创建时间
    server_ctime: Optional[int] = None
    # 文件在服务端修改时间
    server_mtime: Optional[int] = None
    from_type: Optional[int] = None
    # 云端哈希（非文件真实MD5）
    md5: Optional[str] = None
    object_key: Optional[str] = None
    share: Optional[int] = None
    owner_id: Optional[int] = None
    wpfile: Optional[int] = None


class FileMetaThumb(BaseModel):
    """文件元数据图片缩略图信息"""
    # 60x60大小的图片路径
    icon: Optional[str] = None
    # 140x90大小的图片路径
    url1: Optional[str] = None
    # 360x270大小的图片路径
    url2: Optional[str] = None
    # 850x580大小的图片路径
    url3: Optional[str] = None
    # 原质量
    url4: Optional[str] = None


class ImageInfo(BaseModel):
    """图片信息"""
    # 文件在云端的唯一标识
    fs_id: int
    # 文件的绝对路径
    path: str
    # 文件名称
    server_filename: str
    # 文件类型，1 视频、2 音频、3 图片、4 文档、5 应用、6 其他、7 种子
    category: int
    # 是否是目录，0为否，1为是
    isdir: int
    # 文件在服务端创建时间
    server_ctime: Optional[int] = None
    # 文件在服务端修改时间
    server_mtime: Optional[int] = None
    # 文件在客户端修改时间
    local_mtime: Optional[int] = None
    # 文件在客户端创建时间
    local_ctime: Optional[int] = None
    # 云端哈希（非文件真实MD5）
    md5: str
    # 文件大小，单位B
    size: int
    # 图片缩略图，包含四个尺寸
    thumbs: Optional[FileMetaThumb] = None
    from_type: Optional[int] = None
    wpfile: Optional[int] = None
    owner_id: Optional[int] = None
    object_key: Optional[str] = None
    share: Optional[int] = None


class VideoInfo(BaseModel):
    """视频信息"""
    # 文件在云端的唯一标识
    fs_id: int
    # 文件的绝对路径
    path: str
    # 文件名称
    server_filename: str
    # 文件类型，1 视频、2 音频、3 图片、4 文档、5 应用、6 其他、7 种子
    category: int
    # 是否是目录，0为否，1为是
    isdir: int
    # 文件在服务端创建时间
    server_ctime: Optional[int] = None
    # 文件在服务端修改时间
    server_mtime: Optional[int] = None
    # 文件在客户端修改时间
    local_mtime: Optional[int] = None
    # 文件在客户端创建时间
    local_ctime: Optional[int] = None
    # 云端哈希（非文件真实MD5）
    md5: str
    # 文件大小，单位B
    size: int
    # 图片缩略图，包含四个尺寸
    thumbs: Optional[FileMetaThumb] = None
    object_key: Optional[str] = None
    share: Optional[int] = None


class BtInfo(BaseModel):
    """bt信息"""
    # 文件在云端的唯一标识
    fs_id: int
    # 文件的绝对路径
    path: str
    # 文件名称
    server_filename: str
    # 文件类型，1 视频、2 音频、3 图片、4 文档、5 应用、6 其他、7 种子
    category: int
    # 是否是目录，0为否，1为是
    isdir: int
    # 文件在服务端创建时间
    server_ctime: Optional[int] = None
    # 文件在服务端修改时间
    server_mtime: Optional[int] = None
    # 文件在客户端修改时间
    local_mtime: Optional[int] = None
    # 文件在客户端创建时间
    local_ctime: Optional[int] = None
    # 云端哈希（非文件真实MD5）
    md5: str
    # 文件大小，单位B
    size: int
    object_key: Optional[str] = None
    share: Optional[int] = None


class MultimediaFileInfo(BaseModel):
    """MultimediafileApi的listall接口返回文件新"""
    errno: Optional[int] = None
    # 是否还有下一页，0表示无，1表示有
    has_more: bool
    # 当还有下一页时，为下一次查询的起点
    cursor: Optional[int] = None
    # 文件列表
    file_list: Optional[List[FileInfo]] = None


class FileMetaInfo(BaseModel):
    """文件元信息"""
    # 文件在云端的唯一标识ID
    fs_id: int
    # 文件类型，含义如下：1 视频， 2 音乐，3 图片，4 文档，5 应用，6 其他，7 种子
    category: Optional[int] = None
    # 文件的绝对路径
    path: Optional[str] = None
    # 文件下载地址
    dlink: Optional[str] = None
    # 文件名称
    filename: Optional[str] = None
    # 是否为目录，0 文件、1 目录
    isdir: Optional[int] = None
    # 文件的服务器创建Unix时间戳，单位秒
    server_ctime: Optional[int] = None
    # 文件的服务器修改Unix时间戳，单位秒
    server_mtime: Optional[int] = None
    # 文件在客户端修改时间
    local_mtime: Optional[int] = None
    # 文件在客户端创建时间
    local_ctime: Optional[int] = None
    # 文件大小，单位B
    size: Optional[int] = None
    # 云端哈希（非文件真实MD5）
    md5: Optional[str] = None
    # 缩略图URL
    thumbs: Optional[FileMetaThumb] = None
    # 图片高度
    height: Optional[int] = None
    # 图片宽度
    width: Optional[int] = None
    # 图片拍摄时间
    date_taken: Optional[int] = None
    # 图片旋转方向信息
    orientation: Optional[str] = None
    oper_id: Optional[int] = None


class SearchFileInfo(BaseModel):
    """搜索文件信息"""
    # 文件在云端的唯一标识ID
    fs_id: int
    # 文件类型，含义如下：1 视频， 2 音乐，3 图片，4 文档，5 应用，6 其他，7 种子
    category: int
    # 文件的绝对路径
    path: Optional[str] = None
    # 文件名称
    server_filename: str
    # 文件大小，单位B
    size: int
    # 文件在服务端创建时间
    server_ctime: Optional[int] = None
    # 文件在服务端修改时间
    server_mtime: Optional[int] = None
    # 文件在客户端修改时间
    local_mtime: Optional[int] = None
    # 文件在客户端创建时间
    local_ctime: Optional[int] = None
    # 是否为目录，0 文件、1 目录
    isdir: int
    # 缩略图URL
    thumbs: Optional[FileMetaThumb] = None
    # 云端哈希（非文件真实MD5）
    md5: Optional[str] = None
    share: Optional[int] = None
    oper_id: Optional[int] = None
    extent_tinyint1: Optional[int] = None


class SearchResultInfo(BaseModel):
    """搜索结果信息"""
    # 是否有更多
    has_more: bool
    # 内容列表
    content_list: Optional[list] = None
    # 文件列表
    file_list: Optional[List[SearchFileInfo]] = None


class CategoryCountInfo(BaseModel):
    """文件分类数量信息"""
    total: Optional[int] = None
    size: Optional[int] = None
    # 文件数量
    count: int


class CategoryListItem(BaseModel):
    """CategoryList的子项"""
    # 文件在云端的唯一标识ID
    fs_id: int
    # 文件类型，含义如下：1 视频， 2 音乐，3 图片，4 文档，5 应用，6 其他，7 种子
    category: int
    # 文件的绝对路径
    path: Optional[str] = None
    # 文件名称
    server_filename: str
    # 文件大小，单位B
    size: int
    # 文件在服务端创建时间
    server_ctime: Optional[int] = None
    # 文件在服务端修改时间
    server_mtime: Optional[int] = None
    # 文件在客户端修改时间
    local_mtime: Optional[int] = None
    # 文件在客户端创建时间
    local_ctime: Optional[int] = None
    # 是否为目录，0 文件、1 目录
    isdir: int
    # 缩略图URL
    thumbs: Optional[FileMetaThumb] = None
    # 云端哈希（非文件真实MD5）
    md5: Optional[str] = None


class CategoryListResult(BaseModel):
    """CategoryList结果"""
    # 是否还有下一页，0表示无，1表示有
    has_more: bool
    # 当还有下一页时，为下一次查询的起点
    cursor: Optional[int] = None
    # 文件列表
    file_list: Optional[List[CategoryListItem]] = None


class CreatePathResult(BaseModel):
    """获取文件信息返回结果"""
    # 文件在云端的唯一标识ID
    fs_id: int
    # 文件名
    name: str
    # 上传后使用的文件绝对路径
    path: str
    # 文件类型，含义如下：1 视频， 2 音乐，3 图片，4 文档，5 应用，6 其他，7 种子
    category: int
    # 是否为目录，0 文件、1 目录
    isdir: int
    # 文件创建时间
    ctime: int
    # 文件修改时间
    mtime: int


class CopyFileInfo(BaseModel):
    """拷贝文件信息"""
    # 源目录
    source_file: str
    # 目标目录
    dest_path: str
    # 新文件名
    new_filename: str
    # 文件对应的ondup参数, fail(默认，直接返回失败)、newcopy(重命名文件)、overwrite、skip； 高于全局ondup
    ondup: Optional[str] = None


class FileSyncManageResultItem(BaseModel):
    """同步管理结果"""
    # 文件路径
    path: str
    # 当为0时成功
    errno: int


class FileAsyncManageResult(BaseModel):
    """文件异步管理结果"""
    # 当异步执行时，还有taskid
    taskid: Optional[int] = None


class MoveFileInfo(BaseModel):
    """移动文件信息"""
    # 源目录
    source_file: str
    # 目标目录
    dest_path: str
    # 新文件名
    new_filename: str
    # 文件对应的ondup参数, fail(默认，直接返回失败)、newcopy(重命名文件)、overwrite、skip； 高于全局ondup
    ondup: Optional[str] = None


class RenameFileInfo(BaseModel):
    """重命名文件信息"""
    # 源目录
    source_file: str
    # 新文件名
    new_filename: str


class PreCreateResult(BaseModel):
    """预上传结果"""
    # 错误码
    errno: int
    request_id: int
    # 文件的绝对路径
    path: Optional[str] = None
    # 上传唯一ID标识此上传任务
    uploadid: Optional[str] = None
    # 返回类型，系统内部状态字段
    return_type: Optional[int] = None
    # 需要上传的分片序号列表，索引从0开始
    block_list: Optional[list[int]] = None


class SplitUploadResult(BaseModel):
    """分片上传结果"""
    # 错误码
    errno: Optional[int] = None
    request_id: Optional[int] = None
    # 文件切片云端md5
    md5: Optional[str] = None


class ImageExifInfo(BaseModel):
    """图片额外信息"""
    # 如：6
    orientation: Optional[int] = None
    # 如：4032
    width: Optional[int] = None
    # 如：3024
    height: Optional[int] = None
    # 如：0
    recovery: Optional[int] = None
    # 如：2018:09:06 15:58:58
    date_time_original: Optional[datetime]
    # 如：2018:09:06 15:58:58
    date_time_digitized: Optional[datetime]
    # 如：2018:09:06 15:58:58
    date_time: Optional[datetime]
    # 如：iPhone 6s
    model: Optional[str] = None


class UploadCreateResult(BaseModel):
    """上传的第三步：创建文件结果"""
    # 错误码
    errno: Optional[int] = None
    show_msg: Optional[str] = None
    request_id: Optional[int] = None
    # 文件在云端的唯一标识ID
    fs_id: Optional[int] = None
    # 文件的MD5，只有提交文件时才返回，提交目录时没有该值
    md5: Optional[str] = None
    # 文件名
    server_filename: Optional[str] = None
    # 分类类型, 1 视频 2 音频 3 图片 4 文档 5 应用 6 其他 7 种子
    category: Optional[int] = None
    # 上传后使用的文件绝对路径
    path: Optional[str] = None
    name: Optional[str] = None
    # 文件大小，单位B
    size: Optional[int] = None
    # 文件创建时间
    ctime: Optional[int] = None
    # 文件修改时间
    mtime: Optional[int] = None
    # 是否目录，0 文件、1 目录
    isdir: Optional[int] = None
    from_type: Optional[int] = None


class SplitUploadTempInfo(BaseModel):
    """分片上传临时文件"""
    # 文件大小
    file_size: Optional[int] = None
    # 是否是文件夹
    is_dir: Optional[bool] = None
    # 分片信息
    part_info_list: Optional[list[tuple[int, int]]] = None
    # 预上传id
    upload_id: Optional[str] = None
    # 文件各分片MD5数组
    block_list: list[str]
    # 已经上传的分片信息
    uploaded_part_list: Optional[list[tuple[int, int]]] = []


class AudioCategoryInfo(BaseModel):
    # 播单名
    name: Optional[str] = None
    # 播单ID
    mb_id: Optional[int] = None
    # 播单内文件数目
    file_count: Optional[int] = None
    # 播单创建时间
    ctime: Optional[int] = None
    # 播单修改时间
    mtime: Optional[int] = None
    # 播单文件类型，0 音频
    btype: Optional[int] = None
    # 播单子类型，0普通 1音乐 2课程
    bstype: Optional[int] = None


class ListAudioCategoryResult(BaseModel):
    errno: Optional[int] = None
    request_id: Optional[int] = None
    show_msg: Optional[str] = None
    newno: Optional[str] = None
    # 是否还有下一页，0表示无，1表示有
    has_more: Optional[int] = None
    # 播单列表
    data_list: Optional[list[AudioCategoryInfo]] = None


class AudioInfo(BaseModel):
    # 文件在服务端的唯一标识id
    fs_id: int
    # 文件路径
    path: str
    # 文件修改时间
    broadcast_file_mtime: int
    # 以下是文件详细信息
    # 文件名称
    server_filename: Optional[str] = None
    # 是否是文件夹。1是，为文件夹；0否，为文件
    isdir: Optional[str] = None
    # 文件大小
    size: Optional[str] = None
    # 文件类型
    category: Optional[str] = None
    # 服务端哈希（非文件真实MD5）
    md5: Optional[str] = None
    # 文件私密等级
    privacy: Optional[str] = None
    server_atime: Optional[str] = None
    # 文件在服务端创建时间
    server_ctime: Optional[str] = None
    # 文件在服务端修改时间
    server_mtime: Optional[str] = None
    # 文件在客户端创建时间
    local_ctime: Optional[str] = None
    # 文件在客户端修改时间
    local_mtime: Optional[str] = None
    tkbind_id: Optional[str] = None
    owner_type: Optional[str] = None
    share: Optional[str] = None
    real_category: Optional[str] = None
    videotag: Optional[str] = None
    wpfile: Optional[str] = None
    oper_id: Optional[str] = None
    owner_id: Optional[str] = None


class ListAudioResult(BaseModel):
    errno: Optional[int] = None
    request_id: Optional[int] = None
    show_msg: Optional[str] = None
    newno: Optional[str] = None
    # 是否还有下一页，0表示无，1表示有
    has_more: Optional[int] = None
    # 文件列表
    data_list: Optional[list[AudioInfo]] = None


class AdTokenResult(BaseModel):
    # 错误码
    errno: Optional[int] = None
    # 广告播放时长
    adTime: Optional[int] = None
    # 加载广告后返回的合法token，有效期10小时
    adToken: Optional[str] = None
    # 	第二次请求应该在第一次响应后，过ltime秒后再发起
    ltime: Optional[int] = None


class CreateShareInfo(BaseModel):
    # 分享id
    share_id: Optional[str] = None
    # 分享短链
    short_url: Optional[str] = None
    # 分享访问url
    link: Optional[str] = None
    # 分享有效期
    period: Optional[str] = None
    # 分享提取码
    pwd: Optional[str] = None
    # 分享备注
    remark: Optional[str] = None


class CreateShareResult(BaseModel):
    # 错误码
    errno: Optional[int] = None
    # 请求标识
    request_id: Optional[int] = None
    # 错误信息
    show_msg: Optional[str] = None
    # 响应数据
    data: Optional[CreateShareInfo] = None


class VerifySharePwdInfo(BaseModel):
    # 加密的提取码，用于后续操作携带
    spwd: Optional[str] = None


class VerifySharePwdResult(BaseModel):
    # 错误码
    errno: Optional[int] = None
    # 请求标识
    request_id: Optional[int] = None
    # 错误信息
    show_msg: Optional[str] = None
    # 响应数据
    data: Optional[VerifySharePwdInfo] = None


class ShareLinkInfo(BaseModel):
    """分享外链信息"""
    # 分享短链
    short_url: Optional[str] = None
    # 分享id
    share_id: Optional[int] = None
    # 分享者uk
    share_uk: Optional[int] = None
    # 分享备注信息
    remark: Optional[str] = None
    # 分享行为限制
    vdt_limit: Optional[str] = None
    # 分享有效期
    period: Optional[int] = None
    # 分享创建时间
    ctime: Optional[int] = None
    # 分享者归属app_id，提取码验证通过后返回
    appid: Optional[int] = None
    # 分享链接，提取码验证通过后返回
    link: Optional[str] = None
    # 分享状态，提取码验证通过后返回
    status: Optional[int] = None
    # 分享记录最后修改时间，提取码验证通过后返回
    mtime: Optional[int] = None
    # 分享提取码，提取码验证通过后返回
    pwd: Optional[str] = None
    extra: Optional[dict] = None
    file_cnt: Optional[int] = None
    pwd_pass: Optional[bool] = None


class ShareUserInfo(BaseModel):
    """分享者信息"""
    # 分享者用户名，脱敏后
    user_name: Optional[str] = None
    # 分享者头像信息
    user_avatar: Optional[str] = None


class QueryShareInfoData(BaseModel):
    # 分享外链信息
    link_info: Optional[ShareLinkInfo] = None
    # 分享者信息
    share_user_info: Optional[ShareUserInfo] = None


class QueryShareInfoResult(BaseModel):
    # 错误码
    errno: Optional[int] = None
    # 请求标识
    request_id: Optional[int] = None
    # 错误信息
    show_msg: Optional[str] = None
    # 响应数据
    data: Optional[QueryShareInfoData] = None


class QueryShareDataListItem(BaseModel):
    # 文件分类
    category: Optional[int] = None
    # 文件id
    fsid: Optional[int] = None
    # 0 or 1, 文件是否是目录
    isdir: Optional[int] = None
    # 本地创建时间
    local_ctime: Optional[int] = None
    # 本地修改时间
    local_mtime: Optional[int] = None
    # 文件md5
    md5: Optional[str] = None
    # 文件路径
    path: Optional[str] = None
    # 服务端创建时间
    server_ctime: Optional[int] = None
    # 服务端修改时间
    server_mtime: Optional[int] = None
    # 文件名
    server_filename: Optional[str] = None
    # 文件大小
    size: Optional[int] = None


class QueryShareDataListData(BaseModel):
    # 返回的文件列表个数
    count: Optional[int] = None
    # 文件列表
    item_list: Optional[list[QueryShareDataListItem]] = None


class QueryShareDataListResult(BaseModel):
    # 错误码
    errno: Optional[int] = None
    # 请求标识
    request_id: Optional[int] = None
    # 错误信息
    show_msg: Optional[str] = None
    # 响应数据
    data: Optional[QueryShareDataListData] = None


class TransferShareData(BaseModel):
    # 任务id，同步返回 0
    task_id: Optional[str] = None
    # 权益绑定错误号
    tkbind_errno: Optional[int] = None


class TransferShareResult(BaseModel):
    # 错误码
    errno: Optional[int] = None
    # 请求标识
    request_id: Optional[int] = None
    # 错误信息
    show_msg: Optional[str] = None
    # 响应数据
    data: Optional[TransferShareData] = None


class TransferShareTaskQueryDataDescItem(BaseModel):
    """任务文件列表项"""
    from_: Optional[str] = None
    # 转存文件来源path
    # 转存文件保存path
    to: Optional[str] = None
    # 转存文件保存fsid
    to_fs_id: Optional[int] = None


class TransferShareTaskQueryDataDesc(BaseModel):
    """任务结果详情"""
    # 执行成功文件数
    succNum: Optional[int] = None
    num_limit: Optional[int] = None
    num_limit_no_vip: Optional[int] = None
    # 任务文件列表
    item_list: Optional[list[TransferShareTaskQueryDataDescItem]] = None


class TransferShareTaskQueryData(BaseModel):
    task_id: Optional[str] = None
    # 任务进度
    progress: Optional[int] = None
    # 任务状态：fail、running、success、pending（任务排队中，表示任务已经提交，待执行）
    status: Optional[str] = None
    # 任务结果详情
    desc: Optional[TransferShareTaskQueryDataDesc] = None


class TransferShareTaskQueryResult(BaseModel):
    # 错误码
    errno: Optional[int] = None
    # 请求标识
    request_id: Optional[int] = None
    # 错误信息
    show_msg: Optional[str] = None
    # 响应数据
    data: Optional[TransferShareTaskQueryData] = None



from typing import Sequence, Any

from fastapi.encoders import jsonable_encoder
from starlette.requests import Request
from starlette.responses import JSONResponse, Response

from afeng_tools.application_tool.application_models import AppInfo
from afeng_tools.fastapi_tool.core.fastapi_items import ResponseResult, ResponseTemplateResult


def json_resp(error_no: int = 0, message: str | Sequence = 'success', data: Any = None,
              http_status: int = 200) -> JSONResponse:
    """json响应"""
    return JSONResponse(
        status_code=http_status,
        content=jsonable_encoder(ResponseResult(error_no=error_no, message=message, data=data)),
    )


def template_resp(request: Request, resp_result: ResponseTemplateResult, app_info: AppInfo = None) -> Response:
    """模板响应"""
    context_data = resp_result.context_data
    if context_data is None:
        context_data = dict()
    if 'app_info' not in context_data:
        context_data['app_info'] = app_info
    if 'title' not in context_data:
        context_data['title'] = resp_result.title
    if 'message' not in context_data:
        context_data['message'] = resp_result.message
    from afeng_tools.fastapi_tool.fastapi_jinja2_tools import create_template_response
    return create_template_response(request=request,
                                    template_file=resp_result.template_file,
                                    context=context_data)


"""
系统平台工具
"""
import platform


def get_system_bit() -> str:
    """
    获取当前安装版本为x86还是x64
    :return: x64、x86
    """
    architecture_data = platform.architecture()
    if architecture_data[0] == "64bit":
        return "x64"
    elif architecture_data[0] == "32bit":
        return "x86"


def get_system() -> str:
    """
    获取系统信息
    :return: linux、windows、mac
    """
    system_data = platform.system()
    if system_data == 'Linux':
        return 'linux'
    elif system_data == 'Windows':
        return 'windows'
    elif system_data == 'Darwin':
        return 'mac'
    else:
        raise NotImplementedError



from afeng_tools.linux_tool import sh_tools


def run_cmd(cmd_str: str) -> list[tuple]:
    """运行命令"""
    result_list = []
    out_list = sh_tools.run(cmd_str)
    if out_list:
        out_list = list(filter(lambda x: True if x else False, out_list))
        for tmp_line in out_list:
            tmp_info_tuple = tuple(filter(lambda x: True if x else False, tmp_line.strip().split(' ')))
            result_list.append(tmp_info_tuple)
    return result_list


def run_task_list(task_name: str):
    result_list = []
    out_list = run_cmd(f'tasklist|findstr "{task_name}"')
    for tmp_info_tuple in out_list:
        result_list.append({
            'title': tmp_info_tuple[0],
            'pid': tmp_info_tuple[1],
            'name': tmp_info_tuple[2],
            'session_count': tmp_info_tuple[3],
            'memory_usage': tmp_info_tuple[4] + tmp_info_tuple[5],
        })
    return result_list


def run_task_kill(pid: int | str):
    run_cmd(f' taskkill /F /pid {pid}')


if __name__ == '__main__':
    for tmp_task in run_task_list('nginx'):
        run_task_kill(tmp_task.get('pid'))




"""
正则工具
"""
import re
from typing import AnyStr, Any


def high_light_word(content_text: str, keyword: str, high_light_tag: str = 'em') -> str:
    """
    单词高亮
    """
    if content_text:
        re_pattern = f'({re.escape(keyword)})'
        if re.search(re_pattern, content_text, flags=re.RegexFlag.I):
            return re.sub(re_pattern, f'<{high_light_tag}>\\1</{high_light_tag}>', content_text, flags=re.RegexFlag.I)
    return content_text


def get_match_groups(pattern:str, content:str) -> tuple[AnyStr | Any, ...]:
    """
    获取规则匹配的分组内容
    :param pattern: 正则表达式
    :param content: 内容
    :return: tuple或None 匹配的分组内容
    """
    re_match = re.compile(pattern, re.S).search(content)
    if re_match:
        return re_match.groups()


def get_match_first_group(pattern:str, content:str) -> str:
    """
    获取规则匹配的第一个分组内容
    :param pattern: 正则表达式
    :param content: 内容
    :return: 匹配的第一个分组内容
    """
    re_match_tuple = get_match_groups(pattern, content)
    if re_match_tuple:
        return re_match_tuple[0]


if __name__ == '__main__':
    pattern = r'<ToUserName><!\[CDATA\[(.*?)\]\]></ToUserName>'
    content = '<ToUserName><![CDATA[toUser]]></ToUserName>'
    print(get_match_first_group(pattern, content))



"""
playwright工具：pip install pytest-playwright -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
"""

import json

from playwright.sync_api import Page


def _save_request_headers(auth_header_file: str):
    def wrap_call(req):
        all_headers_dict = req.all_headers()
        user_headers = dict()
        user_headers["cookie"] = all_headers_dict.get("cookie")
        user_headers['Host'] = all_headers_dict.get('Host')
        user_headers["referer"] = all_headers_dict.get("referer")
        user_headers["user-agent"] = all_headers_dict.get("user-agent")
        user_headers["authorization"] = all_headers_dict.get("authorization")
        user_headers["origin"] = all_headers_dict.get("origin")
        header_str = json.dumps(user_headers)
        with open(auth_header_file, 'wb') as header_file:
            header_file.write(header_str.encode('utf-8'))
    return wrap_call


def save_headers(page: Page, auth_header_file):
    """保存header"""
    page.on("requestfinished", _save_request_headers(auth_header_file))

def set_headers(page: Page, auth_header_file):
    """设置header"""
    with open(auth_header_file, 'rb') as header_file:
        header_str = header_file.read()
    header_dict = json.loads(header_str)
    page.set_extra_http_headers(header_dict)
    page.reload()

def get_headers(auth_header_file) -> dict:
    """获取header"""
    with open(auth_header_file, 'rb') as header_file:
        header_str = header_file.read()
        return json.loads(header_str)



"""
xpath工具： 安装：pip install lxml -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
"""

from lxml import etree


def get_html_tree(html_file: str):
    """转换html文件为html_tree"""
    parser = etree.HTMLParser(encoding="utf-8")
    return etree.parse(html_file, parser=parser)


def parse_to_html_tree(html_text: str):
    """转换字符串为html_tree"""
    return etree.HTML(html_text)



"""
this module is a hack only in place to allow for setuptools
to use the attribute for the versions

it works only if the backend-path of the build-system section
from pyproject.toml is respected
"""
from __future__ import annotations

import logging
from typing import Callable

from setuptools_scm import Configuration
from setuptools_scm import _types as _t
from setuptools_scm import get_version
from setuptools_scm import git
from setuptools_scm import hg
from setuptools_scm.fallbacks import parse_pkginfo
from setuptools_scm.version import ScmVersion
from setuptools_scm.version import get_local_node_and_date
from setuptools_scm.version import guess_next_dev_version

log = logging.getLogger("setuptools_scm")
# todo: take fake entrypoints from pyproject.toml
try_parse: list[Callable[[_t.PathT, Configuration], ScmVersion | None]] = [
    parse_pkginfo,
    git.parse,
    hg.parse,
    git.parse_archival,
    hg.parse_archival,
]


def parse(root: str, config: Configuration) -> ScmVersion | None:
    for maybe_parse in try_parse:
        try:
            parsed = maybe_parse(root, config)
        except OSError as e:
            log.warning("parse with %s failed with: %s", maybe_parse, e)
        else:
            if parsed is not None:
                return parsed
    else:
        return None


def scm_version() -> str:
    return get_version(
        # relative_to='..',
        parse=parse,
        version_scheme=guess_next_dev_version,
        local_scheme=get_local_node_and_date,
    )


version: str


def __getattr__(name: str) -> str:
    if name == "version":
        global version
        version = scm_version()
        version = '.'.join(version.split('.')[0:3])
        return version
    raise AttributeError(name)




from typing import Optional

from pydantic import BaseModel


class Jscode2sessionResult(BaseModel):
    # 会话密钥
    session_key: Optional[str] = None
    # 用户在开放平台的唯一标识符，若当前小程序已绑定到微信开放平台账号下会返回，详见 UnionID 机制说明。
    unionid: Optional[str] = None
    # 用户唯一标识
    openid: Optional[str] = None
    # 错误码:  【40029】js_code无效  【45011】API 调用太频繁，请稍候再试 【40226】 高风险等级用户，小程序登录拦截 。 【-1】系统繁忙，此时请开发者稍候再试
    errcode: Optional[int] = None
    # 错误信息
    errmsg: Optional[str] = None


class TokenResult(BaseModel):
    # 获取到的凭证
    access_token: Optional[str] = None
    # 凭证有效时间，单位：秒。目前是7200秒之内的值。
    expires_in: Optional[int] = None


class CheckResult(BaseModel):
    # 错误码, 0: ok   87009: 无效的签名
    errcode: Optional[int] = None
    # 错误信息
    errmsg: Optional[str] = None


class GenerateUrlLink(BaseModel):
    # 错误码, 0: ok   87009: 无效的签名
    errcode: Optional[int] = None
    # 错误信息
    errmsg: Optional[str] = None
    # 生成的小程序 URL Link
    url_link: Optional[str] = None


class WxUserInfo(BaseModel):
    nickName: Optional[str] = None
    avatarUrl: Optional[str] = None
    city: Optional[str] = None
    country: Optional[str] = None
    province: Optional[str] = None
    gender: Optional[int] = None
    is_demote: Optional[bool] = None
    language: Optional[str] = None


class WxUserProfile(BaseModel):
    encryptedData: Optional[str] = None
    iv: Optional[str] = None
    rawData: Optional[str] = None
    signature: Optional[str] = None
    userInfo: Optional[WxUserInfo] = None
    app_id: str
    token: str


class WxUserBasicInfo(BaseModel):
    app_id: str
    token: str
    nickName: Optional[str] = None
    avatarUrl: Optional[str] = None



"""
pydantic 工具
- pip install pydantic -i https://pypi.tuna.tsinghua.edu.cn/simple/
"""
from typing import Type, Optional

import pydantic
from pydantic import BaseModel, Field
from pydantic.fields import FieldInfo



def _get_model_properties(orm_model_class: Type, add_flag: bool = False) \
        -> dict[str, tuple[type[BaseModel], FieldInfo]]:
    """通过orm类，获取model类的属性"""
    orm_column_dict = {c.name: c for c in orm_model_class.__table__.columns}
    add_ignore_list = ['id', 'add_time', 'update_time', 'is_enable', 'order_num']
    properties = dict()
    for tmp_field, tmp_column in orm_column_dict.items():
        if add_flag and tmp_field in add_ignore_list:
            continue
        tmp_type = tmp_column.type.python_type
        properties[tmp_field] = (Optional[tmp_type], Field(default=None, title=tmp_column.comment))
    return properties


def create_model_by_po(orm_model_class: Type, add_flag: bool = False) -> type[BaseModel]:
    """通过orm模型创建pydantic模型类"""
    table_name = orm_model_class.__table__.name
    vo_class_name = ''.join([tmp.title() for tmp in table_name.split('_')])
    class_name = f'{vo_class_name}Form'
    if add_flag:
        class_name = f'{vo_class_name}AddForm'
    return pydantic.create_model(
        __model_name=class_name,
        __config__=None,
        __base__=None,
        __module__=__name__,
        __validators__=None,
        __cls_kwargs__=None,
        __slots__=None,
        **_get_model_properties(orm_model_class, add_flag)
    )





from afeng_tools.aliyun_pan_tool.aliyun_pan_share_tools import get_share_info_by_link, list_share_root_files
from afeng_tools.aliyun_pan_tool.aliyun_pan_tools import get_user_info, get_ali_api, get_file_info, list_file, list_path


def user_info_demo():
    ali_api = get_ali_api()
    user_info = get_user_info(ali_api)
    print(user_info.user_name, user_info.nick_name, user_info.phone)


def list_root_file_demo():
    ali_api = get_ali_api()
    file_list = list_file(ali_api)
    for tmp_file in file_list:
        # 打印文件信息
        # 文件id，如：'63b8e37d08f61f64808b4dfdba2c1119ed140464'
        print(tmp_file.file_id)
        # 文件名，如：'电气'
        print(tmp_file.name)
        # 父id，如：root
        print(tmp_file.parent_file_id)
        # 文件类型 folder:文件夹  file：文件
        print(tmp_file.type)
        # 创建时间,如：'2023-01-07T03:14:05.721Z'
        print(tmp_file.created_at)
        # 修改时间,如：'2023-01-07T03:14:05.721Z'
        print(tmp_file.updated_at)


def share_url_demo():
    ali_api = get_ali_api()
    share_msg = '@夏因平典 数学奥林匹克小丛书 https://www.aliyundrive.com/s/Fu8CA5JoRQy/folder/6356a27c044a680bec9b49a6a6711536ef44c01c'
    result = get_share_info_by_link(ali_api, share_msg)
    share_id = result.share_id
    file_list = list_share_root_files(ali_api, share_id)
    for tmp_file in file_list:
        # 打印文件信息
        # 文件id，如：'63b8e37d08f61f64808b4dfdba2c1119ed140464'
        print(tmp_file.file_id)
        # 文件名，如：'电气'
        print(tmp_file.name)
        # 父id，如：root
        print(tmp_file.parent_file_id)
        # 文件类型 folder:文件夹  file：文件
        print(tmp_file.type)
        # 创建时间,如：'2023-01-07T03:14:05.721Z'
        print(tmp_file.created_at)
        # 修改时间,如：'2023-01-07T03:14:05.721Z'
        print(tmp_file.updated_at)


def list_file_demo():
    ali_api = get_ali_api()
    # tmp_file = get_file_info(ali_api, '电气')
    # tmp_file = get_file_info(ali_api, 'Python/Python中文书籍汇总/机器学习实战.pdf')
    tmp_file = list_path(ali_api, '电气')
    print(tmp_file)


if __name__ == '__main__':
    # user_info_demo()
    # list_file_demo()
    # share_url_demo()
    list_file_demo()



"""
ORM的Model工具
"""
from typing import Any

from pydantic import BaseModel
from pydantic.fields import FieldInfo
from sqlalchemy import Column

from afeng_tools.convert_tool import type_convert_tools


def _convert_value_type(field_value: Any, model_column_field: FieldInfo | Column,
                        date_time_format: str) -> Any:
    """转换ORM的Model的值的类型为pydantic模型值"""
    field_type = None
    if isinstance(model_column_field, FieldInfo):
        field_type = model_column_field.annotation
    elif isinstance(model_column_field, Column):
        field_type = model_column_field.type.python_type
    return type_convert_tools.convert_value_to_type(field_value, field_type, date_time_format=date_time_format)


def _convert_orm_value_type(field_value: Any, py_model_field: FieldInfo,
                            date_time_format: str) -> Any:
    """转换ORM的Model的值的类型为pydantic模型值"""
    return _convert_value_type(field_value, py_model_field, date_time_format)


def orm_model_to_py_model(orm_model: Any, py_model_class: type[BaseModel],
                          date_time_format: str = '%Y-%m-%d %H:%M:%S',
                          field_mapping: dict[str, str] = None) -> Any:
    """
    ORM模型转为pydantic的Model
    :param orm_model: orm的model对象
    :param py_model_class: pydantic的Model类
    :param date_time_format: date_time的字符串格式
    :param field_mapping: 字段映射集{ orm的model字段名: pydantic的Model字段名},示例：{'category_code':'category_name'}
    :return:  pydantic的Model对象
    """
    orm_dict = {c.name: getattr(orm_model, c.name) for c in orm_model.__table__.columns}
    py_model_fields = py_model_class.model_fields
    convert_dict = dict()
    for tmp_field, tmp_value in orm_dict.items():
        if tmp_field in py_model_fields and tmp_value is not None:
            convert_dict[tmp_field] = _convert_orm_value_type(tmp_value, py_model_fields.get(tmp_field),
                                                              date_time_format)
        if field_mapping and tmp_field in field_mapping:
            convert_dict[field_mapping.get(tmp_field)] = _convert_orm_value_type(tmp_value,
                                                                                 py_model_fields.get(
                                                                                     field_mapping.get(tmp_field)),
                                                                                 date_time_format)
    return py_model_class(**convert_dict)


def _convert_py_value_type(field_value: Any, orm_model_column: Column,
                           date_time_format: str) -> Any:
    """转换pydantic的Model的值的类型为ORM模型值"""
    return _convert_value_type(field_value, orm_model_column, date_time_format)


def py_model_to_orm_model(py_model: BaseModel, orm_model_class: type,
                          date_time_format: str = '%Y-%m-%d %H:%M:%S',
                          field_mapping: dict[str, str] = None):
    """
    pydantic的Model转为ORM模型
    :param py_model: pydantic的Model对象
    :param orm_model_class: orm的model类
    :param date_time_format: date_time的字符串格式
    :param field_mapping: 字段映射集{ pydantic的Model字段名：orm的model字段名},示例：{'category_title':'sub_title'}
    :return: orm的model对象
    """
    py_dict = py_model.model_dump()
    orm_column_dict = {c.name: c for c in orm_model_class.__table__.columns}
    orm_dict = dict()
    for tmp_field, tmp_value in py_dict.items():
        if tmp_value is not None and tmp_field in orm_column_dict:
            orm_column = orm_column_dict.get(tmp_field)
            orm_dict[tmp_field] = _convert_py_value_type(tmp_value, orm_column, date_time_format)
        if field_mapping and tmp_field in field_mapping:
            orm_dict[field_mapping.get(tmp_field)] = _convert_orm_value_type(tmp_value,
                                                                             orm_column_dict.get(
                                                                                 field_mapping.get(tmp_field)),
                                                                             date_time_format)
    return orm_model_class(**orm_dict)


def orm_model_to_dict(orm_model: Any, skip_none: bool = False):
    """
    orm的Model转为json
    :param orm_model: orm的Model对象
    :param skip_none: 是否跳过None值
    :return: dict数据
    """
    orm_dict = {c.name: getattr(orm_model, c.name) for c in orm_model.__table__.columns}
    if skip_none:
        result_dict = dict()
        for tmp_key, tmp_value in orm_dict.items():
            if tmp_value is not None:
                result_dict[tmp_key] = tmp_value
        return result_dict
    return orm_dict



import os.path
import subprocess
import sys
from typing import Union, Type

from scrapy import Spider
from scrapy.cmdline import execute
from scrapy.crawler import CrawlerProcess, Crawler
from scrapy.utils.project import get_project_settings


def run_spider(spider_dir: str, spider_name: str, output_file: str = None, is_reactor: bool = True, is_wait: bool = False, **kwargs):
    sys.path.append(spider_dir)
    tmp_cwd = os.getcwd()
    os.chdir(spider_dir)
    cmd_list = ['scrapy', 'crawl', spider_name]
    if kwargs:
        for arg_name, arg_value in kwargs.items():
            cmd_list.append('-a')
            cmd_list.append(f'{arg_name}={arg_value}')
    if output_file:
        cmd_list.append('-o')
        cmd_list.append(output_file)
    if is_reactor:
        if is_wait:
            subprocess.check_output(cmd_list)
        else:
            subprocess.Popen(cmd_list, shell=True)
    else:
        execute(cmd_list)
    os.chdir(tmp_cwd)


def run_crawler(crawler_or_spider_cls: Union[Type[Spider], str, Crawler], crawler_dir: str = None, **kwargs):
    """
    运行爬虫（等待结果）
    :param crawler_or_spider_cls:爬虫类
    :param crawler_dir: 爬虫路径
    :param kwargs:参数
    :return:
    """
    sys.path.append(crawler_dir)
    tmp_cwd = os.getcwd()
    os.chdir(crawler_dir)
    settings = get_project_settings()
    crawler_process = CrawlerProcess(settings)
    crawler_process.crawl(crawler_or_spider_cls, **kwargs)
    # 启动爬虫
    crawler_process.start()
    os.chdir(tmp_cwd)


if __name__ == '__main__':
    spider_path = os.path.dirname(os.path.abspath(__file__))
    run_spider(spider_path, 'quotes')



"""同步http工具"""
import httpx


class SyncHttp:
    @classmethod
    def get(cls, url, params=None, headers=None, cookies=None, proxies=None):
        return httpx.get(url, params=params, headers=headers, cookies=cookies, proxies=proxies)

    @classmethod
    def post(cls, url, data=None, headers=None, cookies=None, proxies=None):
        return httpx.post(url=url, data=data, headers=headers, cookies=cookies, proxies=proxies)

    @classmethod
    def get_cookie(cls, url, params=None, headers=None, cookies=None, proxies=None):
        resp = cls.get(url, params=params, headers=headers, cookies=cookies, proxies=proxies)
        return resp.cookies



"""
东方财富Http请求工具类
"""
import json
import os.path
import re
from pathlib import Path

import requests

from afeng_tools.datetime_tool.datetime_tools import get_timestamp
from afeng_tools.east_money_tool.config.web_url_config import get_url
from afeng_tools.file_tool.tmp_file_tools import get_user_tmp_dir
from afeng_tools.http_tool.decorator.http_auto_retry_tools import auto_retry
from afeng_tools.math_tool import random_tools
from afeng_tools.playwright_tool.http_cookie_tools import get_cookies
from afeng_tools.playwright_tool.http_header_tools import get_headers
from afeng_tools.playwright_tool.playwright_tools import refresh_cookie_and_header_file

WORK_PATH = get_user_tmp_dir()

def load_auth_file(cookie_file_name, header_file_name, refresh_url) -> tuple:
    save_path = os.path.join(WORK_PATH, 'auth-file')
    os.makedirs(save_path, exist_ok=True)
    cookie_file = os.path.join(save_path, cookie_file_name)
    header_file = os.path.join(save_path, header_file_name)
    if not os.path.exists(cookie_file) or not os.path.exists(header_file):
        refresh_cookie_and_header_file(refresh_url=refresh_url,
                                       cookie_file=cookie_file, header_file=header_file)
    return cookie_file, header_file


@auto_retry(retry_count=3)
def get_datacenter_data(stock_code: str, opt_dict: dict, cookie_file: str, header_file: str):
    url = 'https:' + get_url('datacenter') + 'api/data/v1/get'
    params = {
        'callback': f'jQuery{random_tools.random_number_str(length=21)}_{get_timestamp()}',
        'filter': "(SECURITY_CODE=\"" + stock_code + "\")",
        'columns': 'ALL',
        'source': 'WEB',
        'client': 'WEB',
    }
    params.update(opt_dict)
    file_path = Path(__file__).parent
    cookies = get_cookies(os.path.join(file_path, cookie_file))
    headers = get_headers(os.path.join(file_path, header_file))
    response = requests.get(url=url, params=params, cookies=cookies, headers=headers)
    if response.status_code == 200:
        response.encoding = 'utf-8'
        response_text = response.text
        re_match = re.match(f'^{params["callback"]}\((.*)\);', response_text)
        if re_match:
            json_str = re_match.group(1)
            return json.loads(json_str)


@auto_retry(retry_count=3)
def http_get(data_url: str, params: dict, cookie_file: str, header_file: str, jsonp: bool = False):
    jsonp_field = 'callback'
    if jsonp:
        if 'cb' in params:
            jsonp_field = 'cb'
        params[jsonp_field] = f'jQuery{random_tools.random_number_str(length=21)}_{get_timestamp()}'
    file_path = Path(__file__).parent
    cookies = get_cookies(os.path.join(file_path, cookie_file))
    headers = get_headers(os.path.join(file_path, header_file))
    response = requests.get(url=data_url, params=params, cookies=cookies, headers=headers)
    if response.status_code == 200:
        response.encoding = 'utf-8'
        response_text = response.text
        if jsonp:
            re_match = re.match(f'^{params[jsonp_field]}\((.*)\);', response_text)
            if re_match:
                json_str = re_match.group(1)
                return json.loads(json_str)
        else:
            return json.loads(response_text)

if __name__ == '__main__':
    pass



from afeng_tools.re_tool.re_tools import get_match_first_group


def parse_event(msg):
    return get_match_first_group(r'<Event><!\[CDATA\[(.*?)\]\]></Event>', msg)


def parse_event_key(msg):
    return get_match_first_group(r'<EventKey><!\[CDATA\[(.*?)\]\]></EventKey>', msg)


def parse_ticket(msg):
    return get_match_first_group(r'<Ticket><!\[CDATA\[(.*?)\]\]></Ticket>', msg)


def parse_location_x(msg):
    return get_match_first_group(r'<Longitude>(.*?)</Longitude>', msg)


def parse_location_y(msg):
    return get_match_first_group(r'<Latitude>(.*?)</Latitude>', msg)


def parse_location_precision(msg):
    return get_match_first_group(r'<Precision>(.*?)</Precision>', msg)



"""
东方财富的URL配置
"""
import math
import random

rnd = math.floor(random.random() * 99 + 1)
rnd2 = math.floor(random.random() * 99 + 1)

# 开发URL
development_urls = {
    'dataurl': '//reportapi.uat.emapd.com/',
    'quoteurl': '//push2.eastmoney.com/',
    'quotehisurl': '//push2his.eastmoney.com/',
    'dcfmurl': '//dcfm.eastmoney.com/',
    'anoticeurl': '//np-anotice-stock.eastmoney.com/',
    'cnoticeurl': '//np-cnotice-stock.eastmoney.com/',
    'datacenter': '//datacenter-web.eastmoney.com/',
    'soapi': '//searchapi.eastmoney.com/',
    'cmsdataapi': '//cmsdataapi.eastmoney.com/',
    'newsinfo': '//newsinfo.eastmoney.com/',
    # 老资讯接口
    'old_zixun_api': "//cmsdataapi.eastmoney.com/",
    # 新资讯接口
    'new_zixun_api': "//np-listapi.eastmoney.com/",
    # 公告
    'new_notice_api': "//np-anotice-stock.eastmoney.com/",
    # 行情
    'quote_api': "//push2.eastmoney.com/",
    # 行情推送接口
    'quote_push_api': "//" + str(rnd) + ".push2.eastmoney.com/",
    # 行情历史数据接口
    'quote_history_api': "//push2his.eastmoney.com/",
    # 行情历史数据推送接口
    'quote_history_push_api': "//" + str(rnd2) + ".push2his.eastmoney.com/",
    # 快讯
    'kuaixun_api': "//newsinfo.eastmoney.com/",
    # 老的数据接口
    'old_datacenter': "//dcfm.eastmoney.com/",
    # 静态行情图-分时图
    'chart_time': "//webquotepic.eastmoney.com/",
    # 静态行情图-延迟分时图
    'chart_time_delay': "//delaywebquotepic.eastmoney.com/",
    # 静态行情图-K线图
    'chart_kline': "//webquoteklinepic.eastmoney.com/",
    # 静态行情图-延迟K线图
    'chart_kline_delay': "//delaywebquoteklinepic.eastmoney.com/",
    # 研报
    'report_api': "//reportapi.eastmoney.com/",
    # 路演api
    'roadshow_api': "//list.lvb.eastmoney.com/",
    # 数据接口
    'datainterface': "//datainterface.eastmoney.com/",
    # 数据4期
    'datacenter4': "//datacenter-web.eastmoney.com/",
    # 期货接口
    'qihuo_api': "//futsseapi.eastmoney.com/",
    # 期货推送接口
    'qihuo_sse_api': "//" + str(rnd2) + ".futsseapi.eastmoney.com/",
    # 期货静态接口
    'qihuo_static': "//futsse-static.eastmoney.com/",
    # 基金接口
    'fund_api': "//fundwebapi.eastmoney.com/",
    # 盘口异动接口
    'changes_api': "//push2ex.eastmoney.com/"
}

# 测试URL
test_urls = {
    'dataurl': '//reportapi.uat.emapd.com/',
    'quoteurl': '//push2.eastmoney.com/',
    'quotehisurl': '//push2his.eastmoney.com/',
    'dcfmurl': '//dcfm.eastmoney.com/',
    'anoticeurl': '//np-anotice-stock-test.eastmoney.com/',
    'cnoticeurl': '//np-cnotice-stock-test.emapd.com/',
    'datacenter': '//testdatacenter.eastmoney.com/',
    'soapi': '//searchapi.eastmoney.com/',
    'cmsdataapi': '//cmsdataapi.eastmoney.com/',
    'newsinfo': '//newsinfo.eastmoney.com/',
    # 老资讯接口
    'old_zixun_api': "//cmsdataapi.eastmoney.com/",
    # 新资讯接口
    'new_zixun_api': "//np-listapi.eastmoney.com/",
    # 公告
    'new_notice_api': "//np-anotice-stock-test.eastmoney.com/",
    # 行情
    'quote_api': "//push2test.eastmoney.com/",
    # 行情推送接口
    'quote_push_api': "//push2test.eastmoney.com/",
    # 行情历史数据接口
    'quote_history_api': "//push2test.eastmoney.com/",
    # 行情历史数据推送接口
    'quote_history_push_api': "//push2test.eastmoney.com/",
    # 快讯
    'kuaixun_api': "//newsinfo.eastmoney.com/",
    # 老的数据接口
    'old_datacenter': "//dcfm.eastmoney.com/",
    # 静态行情图-分时图
    'chart_time': "http://61.129.249.32:8870/",
    # 静态行情图-延迟分时图
    'chart_time_delay': "http://61.129.249.32:8870/",
    # 静态行情图-K线图
    'chart_kline': "http://61.129.249.32:8871/",
    # 静态行情图-延迟K线图
    'chart_kline_delay': "http://61.129.249.32:8871/",
    # 研报
    'report_api': "//reportapi.eastmoney.com/",
    # 路演api
    'roadshow_api': "//list-qas.lvb.eastmoney.com/",
    # 数据接口
    'datainterface': "//datainterface.eastmoney.com/",
    # 数据4期
    'datacenter4': "//testdatacenter.eastmoney.com/",
    # 期货接口
    'qihuo_api': "//futssetest.eastmoney.com/",
    # 期货推送接口
    'qihuo_sse_api': "http://futssetest.eastmoney.com/",
    # 期货静态接口
    'qihuo_static': "//static.futssetest.eastmoney.com/",
    # 基金接口
    'fund_api': "//fundwebapi.eastmoney.com/",
    # 盘口异动接口
    'changes_api': "//push2ex.eastmoney.com/"
}

# 测试1URL
test1_urls = {
    # 老资讯接口
    'old_zixun_api': "//cmsdataapi.eastmoney.com/",
    # 新资讯接口
    'new_zixun_api': "//np-listapi.eastmoney.com/",
    # 公告
    'new_notice_api': "//np-anotice-stock-test.eastmoney.com/",
    # 行情
    'quote_api': "//push2test.eastmoney.com/",
    # 行情推送接口
    'quote_push_api': "//push2test.eastmoney.com/",
    # 行情历史数据接口
    'quote_history_api': "//push2test.eastmoney.com/",
    # 行情历史数据推送接口
    'quote_history_push_api': "//push2test.eastmoney.com/",
    # 快讯
    'kuaixun_api': "//newsinfo.eastmoney.com/",
    # 老的数据接口
    'old_datacenter': "//dcfm.eastmoney.com/",
    # 静态行情图-分时图
    'chart_time': "http://61.129.249.32:8870/",
    # 静态行情图-延迟分时图
    'chart_time_delay': "http://61.129.249.32:8870/",
    # 静态行情图-K线图
    'chart_kline': "http://61.129.249.32:8871/",
    # 静态行情图-延迟K线图
    'chart_kline_delay': "http://61.129.249.32:8871/",
    # 研报
    'report_api': "//reportapi.eastmoney.com/",
    # 路演api
    'roadshow_api': "//list-qas.lvb.eastmoney.com/",
    # 数据接口
    'datainterface': "//datainterface.eastmoney.com/",
    # 数据4期
    'datacenter4': "//testdatacenter.eastmoney.com/",
    # 期货接口
    'qihuo_api': "//futssetest1.eastmoney.com/",
    # 期货推送接口
    'qihuo_sse_api': "http://futssetest1.eastmoney.com/",
    # 期货静态接口
    'qihuo_static': "//static.futssetest.eastmoney.com/",
    # 基金接口
    'fund_api': "//fundwebapi.eastmoney.com/",
    # 盘口异动接口
    'changes_api': "//push2ex.eastmoney.com/"
}

# 灰度测试URL
gray_urls = {
    'dataurl': '//reportapi.eastmoney.com/',
    'quoteurl': '//push2.eastmoney.com/',
    'quotehisurl': '//push2his.eastmoney.com/',
    'dcfmurl': '//dcfm.eastmoney.com/',
    'anoticeurl': '//np-anotice-stock.eastmoney.com/',
    'cnoticeurl': '//np-cnotice-stock.eastmoney.com/',
    'datacenter': '//graydatacenter.eastmoney.com/',
    'soapi': '//searchapi.eastmoney.com/',
    'cmsdataapi': '//cmsdataapi.eastmoney.com/',
    'newsinfo': '//newsinfo.eastmoney.com/'
}

# 生产URL
production_urls = {
    'dataurl': '//reportapi.eastmoney.com/',
    'quoteurl': '//push2.eastmoney.com/',
    'quotehisurl': '//push2his.eastmoney.com/',
    'dcfmurl': '//dcfm.eastmoney.com/',
    'anoticeurl': '//np-anotice-stock.eastmoney.com/',
    'cnoticeurl': '//np-cnotice-stock.eastmoney.com/',
    'datacenter': '//datacenter-web.eastmoney.com/',
    'soapi': '//searchapi.eastmoney.com/',
    'cmsdataapi': '//cmsdataapi.eastmoney.com/',
    'newsinfo': '//newsinfo.eastmoney.com/',
    # 老资讯接口
    'old_zixun_api': "//cmsdataapi.eastmoney.com/",
    # 新资讯接口
    'new_zixun_api': "//np-listapi.eastmoney.com/",
    # 公告
    'new_notice_api': "//np-anotice-stock.eastmoney.com/",
    # 行情
    'quote_api': "//push2.eastmoney.com/",
    # 行情推送接口
    'quote_push_api': "//" + str(rnd) + ".push2.eastmoney.com/",
    # 行情历史数据接口
    'quote_history_api': "//push2his.eastmoney.com/",
    # 行情历史数据推送接口
    'quote_history_push_api': "//" + str(rnd2) + ".push2his.eastmoney.com/",
    # 快讯
    'kuaixun_api': "//newsinfo.eastmoney.com/",
    # 老的数据接口
    'old_datacenter': "//dcfm.eastmoney.com/",
    # 静态行情图-分时图
    'chart_time': "//webquotepic.eastmoney.com/",
    # 静态行情图-延迟分时图
    'chart_time_delay': "//delaywebquotepic.eastmoney.com/",
    # 静态行情图-K线图
    'chart_kline': "//webquoteklinepic.eastmoney.com/",
    # 静态行情图-延迟K线图
    'chart_kline_delay': "//delaywebquoteklinepic.eastmoney.com/",
    # 研报
    'report_api': "//reportapi.eastmoney.com/",
    # 路演api
    'roadshow_api': "//list.lvb.eastmoney.com/",
    # 数据接口
    'datainterface': "//datainterface.eastmoney.com/",
    # 数据4期
    'datacenter4': "//datacenter-web.eastmoney.com/",
    # 期货接口
    'qihuo_api': "//futsseapi.eastmoney.com/",
    # 期货推送接口
    'qihuo_sse_api': "//" + str(rnd2) + ".futsseapi.eastmoney.com/",
    # 期货静态接口
    'qihuo_static': "//futsse-static.eastmoney.com/",
    # 基金接口
    'fund_api': "//fundwebapi.eastmoney.com/",
    # 盘口异动接口
    'changes_api': "//push2ex.eastmoney.com/"
}


def get_url(url_key):
    # 生产环境
    env = 'dev'
    if env == 'dev':
        return development_urls[url_key]
    elif env == 'test':
        return test_urls[url_key]
    elif env == 'test1':
        return test1_urls[url_key]
    elif env == 'gray':
        return gray_urls[url_key]
    return production_urls[url_key]



from afeng_tools.fastapi_tool.common.enum import IconTypeEnum


def get_icon_code(icon_type: IconTypeEnum, icon_value: str, alt: str = '') -> str | None:
    """获取icon代码"""
    if icon_type == IconTypeEnum.svg_icon:
        return icon_value
    elif icon_type == IconTypeEnum.resource_icon:
        if icon_value:
            return f'<img data-src="/resource/public/{icon_value}" alt="{alt}"/>'
        else:
            return None
    else:
        return f'<img data-src="{icon_value}" data-type="image" referrer="no-referrer" referrerpolicy="no-referrer" alt="{alt}" title="{alt}"/>'



from sqlalchemy import Column, DateTime, Boolean, Float, Integer, func, BigInteger

from afeng_tools.sqlalchemy_tool.core.sqlalchemy_db_tools import Base
from afeng_tools.sqlalchemy_tool.core.sqlalchemy_meta_class import ModelMetaClass


class Model(Base, metaclass=ModelMetaClass):
    """模型根类"""
    __abstract__ = True

    id = Column(BigInteger, comment='主键', primary_key=True, index=True, autoincrement=True)
    add_time = Column(DateTime, comment='添加时间', default=func.now())
    update_time = Column(DateTime, comment='修改时间', default=func.now(), onupdate=func.now())
    is_enable = Column(Boolean, comment='是否可用', default=True)
    order_num = Column(Float, comment='排序值', default=100)



import os.path
import zipfile

import pyzipper

from afeng_tools.file_tool.file_tools import list_files


def run_pwd_zip(zip_file: str, file_list: list[tuple[str, str]], password: str = None):
    """
    zip加密压缩：将指定的多个文件压缩到一个 zip 文件
    :param zip_file: zip文件
    :param file_list: 待压缩的文件列表'
    :param password: 加密密码
    :return: 压缩后的zip文件
    """
    with pyzipper.AESZipFile(zip_file, "w", compression=pyzipper.ZIP_LZMA, encryption=pyzipper.WZ_AES) as z:
        # 设置压缩文件的密码
        if password:
            z.setpassword(password.encode('utf-8'))
        for arc_name, tmp_file in file_list:
            if os.path.isfile(tmp_file):
                z.write(tmp_file, arcname=arc_name)


def run_pwd_unzip(zip_file: str, unzip_path: str, password: str = None) -> list[str]:
    """
    解压zip
    :param zip_file: zip文件
    :param unzip_path: 解压到的地址
    :param password: 加密密码
    :return: 压缩文件列表
    """
    with pyzipper.AESZipFile(zip_file, "r") as z:
        if password:
            z.extractall(unzip_path, pwd=password.encode('utf-8'))
        else:
            # 设置解压的地址
            z.extractall(unzip_path)
        return z.namelist()


def run_zip(zip_file: str, file_list: list[tuple[str, str]], password: str = None):
    """
    zip压缩：将指定的多个文件压缩到一个 zip 文件
    :param zip_file: zip文件
    :param file_list: 待压缩的文件列表'
    :param password: 加密密码
    :return: 压缩后的zip文件
    """
    if password:
        run_pwd_zip(zip_file, file_list=file_list, password=password)
    else:
        with zipfile.ZipFile(zip_file, "w") as z:
            for arc_name, tmp_file in file_list:
                if os.path.isfile(tmp_file):
                    z.write(tmp_file, arcname=arc_name)


def run_zip_dir(zip_file: str, dir_path: str, password: str = None):
    """
    zip压缩：将指定的多个文件压缩到一个 zip 文件
    :param zip_file: zip文件
    :param dir_path: 待压缩的文件列表
    :param password: 加密密码
    :return: 压缩后的zip文件
    """
    relative_list, absolute_list = list_files(dir_path)
    run_zip(zip_file, [(tmp, absolute_list[i]) for i, tmp in enumerate(relative_list)], password=password)


def run_unzip(zip_file: str, unzip_path: str, password: str = None) -> list[str]:
    """
    解压zip
    :param zip_file: zip文件
    :param unzip_path: 解压到的地址
    :param password: 加密密码
    :return: 压缩文件列表
    """
    if password:
        run_pwd_unzip(zip_file, unzip_path=unzip_path, password=password)
    else:
        with zipfile.ZipFile(zip_file, "r") as z:
            z.extractall(unzip_path)
            return z.namelist()


if __name__ == '__main__':
    book_path = r'C:\迅雷云盘'
    run_zip_dir(r'C:\迅雷云盘\test.zip', f'C:\迅雷云盘', password='123456')
    run_unzip(r'C:\迅雷云盘\test.zip', r'C:\迅雷云盘\test01', password='123456')



"""
东方财富行情QuoteItem工具类
"""
from afeng_tools.east_money_tool.quote_info.quote_info_config import one_item_config
from afeng_tools.east_money_tool.quote_info.quote_models import QuoteItem


def one_item_fields(item_list: list[QuoteItem]):
    fids = []
    for tmp in item_list:
        fids.extend(tmp.fid_list)
    fids = filter(lambda x: x != 'f531' and x != 'f532', fids)
    return ','.join(fids)


if __name__ == '__main__':
    item_fields = ['名称','扩位简称','市场','代码','行情代码','最新价','最后价格','涨跌幅','涨跌额','股票标识','二级分类','行情统一链接','今开','昨收','最高价','最低价','成交量','盘后成交量','成交量带手','成交额','盘后成交额','是否同股同权','是否表决权差异','注册股本','发行股本','是否盈利','买入价或买一价','买二价','买三价','买四价','买五价','买一量','买二量','买三量','买四量','买五量',
'卖出价或卖一价','卖二价','卖三价','卖四价','卖五价','卖一量','卖二量','卖三量','卖四量','卖五量','买一差量','买二差量','买三差量','买四差量','买五差量',
'卖一差量','卖二差量','卖三差量','卖四差量','卖五差量','内盘','外盘','振幅','量比','交易时间带星期','总股本','流通股本','换手率','每股收益TTM','总市值','市净率','市盈率TTM','市盈率动态','市盈率静态','每股净资产','均价','流通市值','流通市值_短','交易状态','涨停价','跌停价','总市值','委比','委差','异常交易状态','可转债代码或正股代码','是否注册制','是否具有协议控制架构','AB股对应的代码','AB股对应的市场','AH股对应的代码','AH股对应的市场','A股对应GDR或GDR对应A股的代码','A股对应GDR或GDR对应A股的市场','A瑞士对应的市场','A瑞士对应的代码']
    item_list = []
    for tmp_field in item_fields:
        item_list.append(one_item_config[tmp_field])
    fids = one_item_fields(item_list)
    print(fids)



from afeng_tools.fastapi_tool.common.po_service.link_po_service import LinkPoService
from afeng_tools.fastapi_tool.common.service import icon_service
from afeng_tools.fastapi_tool.common.service.base_service import BaseService
from afeng_tools.pydantic_tool.model.common_models import LinkItem


class LinkService(BaseService):
    """
    使用示例：link_service = LinkService(app_info.db_code, LinkInfoPo)
    """

    po_service_type = LinkPoService

    @classmethod
    def convert_po_2_item(cls, data_list: list) -> list[LinkItem]:
        return [LinkItem(
            href=tmp.link_href,
            title=tmp.title,
            desc=tmp.description,
            image=icon_service.get_icon_code(tmp.icon_type, tmp.icon_value, tmp.title),
        ) for tmp in data_list] if data_list else []



import os
from pathlib import Path
from typing import Any

from afeng_tools.application_tool.settings_enum import SettingsKeyEnum
from afeng_tools.file_tool import yaml_tools


def get_settings_item_value_list(yaml_config: dict, settings_key: str | SettingsKeyEnum) -> list[Any]:
    """获取某个配置项的值列表"""
    item_key = settings_key
    if isinstance(settings_key, SettingsKeyEnum):
        item_key = settings_key.value.name
    if yaml_config.get(item_key):
        item_value = yaml_config.get(item_key)
        item_value_list = []
        if isinstance(item_value, list):
            item_value_list = item_value
        else:
            if isinstance(item_value, str) and ',' in item_value:
                item_value_list = item_value.split(',')
            else:
                item_value_list.append(item_value)
        return item_value_list


def _tree_read_config(config: dict, prefix_key: str = None):
    """
    递归读取配置
    :param config: 配置字典
    :param prefix_key: 前缀键
    :return:
    """
    result = dict()
    if config and isinstance(config, dict):
        for tmp_key in config.keys():
            key = prefix_key + '.' + tmp_key if prefix_key else tmp_key
            value = config.get(tmp_key)
            if value and isinstance(value, dict):
                child_result = _tree_read_config(value, key)
                result.update(child_result)
            else:
                result[key] = value
    return result


def read_application_config(yaml_file) -> dict[str, Any]:
    """
    读取application配置
    :param yaml_file: yaml配置文件
    :return: 返回配置字典，如：app.name
    """
    yaml_config = yaml_tools.read_yaml(yaml_file)
    return _tree_read_config(yaml_config)


def read_all_config(yaml_file) -> dict[str, Any]:
    """
    读取所有的配置
    :param yaml_file: yaml配置文件
    :return: 返回配置字典，如：app.name
    """
    yaml_config = read_application_config(yaml_file)
    active_list = get_settings_item_value_list(yaml_config, SettingsKeyEnum.app_active)
    if active_list:
        for tmp_active in active_list:
            active_file_name = f'{os.path.split(yaml_file)[1].rsplit(".", maxsplit=1)[0]}-{tmp_active}{os.path.splitext(yaml_file)[1]}'
            yaml_config.update(read_application_config(os.path.join(Path(yaml_file).parent, active_file_name)))
    return yaml_config



class BaseService:
    po_service_type: type = None

    def __init__(self, db_code: str, po_model_type: type, app_code: str = None):
        self.app_code = app_code
        self.db_code = db_code
        self.po_model_type = po_model_type
        if self.po_service_type:
            self.po_service = self.po_service_type(self.db_code, self.po_model_type)



"""
    xpan

    xpanapi  # noqa: E501

    The version of the OpenAPI document: 0.1
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from openapi_client.api_client import ApiClient, Endpoint as _Endpoint
from openapi_client.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
# from openapi_client.model.filecreateresponse import Filecreateresponse
# from openapi_client.model.fileprecreateresponse import Fileprecreateresponse


class FileuploadApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client
        self.pcssuperfile2_endpoint = _Endpoint(
            settings={
                'response_type': (dict,),
                'auth': [],
                'endpoint_path': '/rest/2.0/pcs/superfile2?method=upload&openapi=xpansdk',
                'operation_id': 'pcssuperfile2',
                'http_method': 'POST',
                'servers': [
                    {
                        'url': "https://d.pcs.baidu.com",
                        'description': "No description provided",
                    },
                ]
            },
            params_map={
                'all': [
                    'access_token',
                    'partseq',
                    'path',
                    'uploadid',
                    'type',
                    'file',
                ],
                'required': [
                    'access_token',
                    'partseq',
                    'path',
                    'uploadid',
                    'type',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'access_token':
                        (str,),
                    'partseq':
                        (str,),
                    'path':
                        (str,),
                    'uploadid':
                        (str,),
                    'type':
                        (str,),
                    'file':
                        (file_type,),
                },
                'attribute_map': {
                    'access_token': 'access_token',
                    'partseq': 'partseq',
                    'path': 'path',
                    'uploadid': 'uploadid',
                    'type': 'type',
                    'file': 'file',
                },
                'location_map': {
                    'access_token': 'query',
                    'partseq': 'query',
                    'path': 'query',
                    'uploadid': 'query',
                    'type': 'query',
                    'file': 'form',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'text/html;charset=utf8'
                ],
                'content_type': [
                    'multipart/form-data'
                ]
            },
            api_client=api_client
        )
        self.xpanfilecreate_endpoint = _Endpoint(
            settings={
                'response_type': (dict,),
                'auth': [],
                'endpoint_path': '/rest/2.0/xpan/file?method=create&openapi=xpansdk',
                'operation_id': 'xpanfilecreate',
                'http_method': 'POST',
                'servers': [
                    {
                        'url': "https://pan.baidu.com",
                        'description': "No description provided",
                    },
                ]
            },
            params_map={
                'all': [
                    'access_token',
                    'path',
                    'isdir',
                    'size',
                    'uploadid',
                    'block_list',
                    'rtype',
                ],
                'required': [
                    'access_token',
                    'path',
                    'isdir',
                    'size',
                    'uploadid',
                    'block_list',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'access_token':
                        (str,),
                    'path':
                        (str,),
                    'isdir':
                        (int,),
                    'size':
                        (int,),
                    'uploadid':
                        (str,),
                    'block_list':
                        (str,),
                    'rtype':
                        (int,),
                },
                'attribute_map': {
                    'access_token': 'access_token',
                    'path': 'path',
                    'isdir': 'isdir',
                    'size': 'size',
                    'uploadid': 'uploadid',
                    'block_list': 'block_list',
                    'rtype': 'rtype',
                },
                'location_map': {
                    'access_token': 'query',
                    'path': 'form',
                    'isdir': 'form',
                    'size': 'form',
                    'uploadid': 'form',
                    'block_list': 'form',
                    'rtype': 'form',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json; charset=UTF-8'
                ],
                'content_type': [
                    'application/x-www-form-urlencoded'
                ]
            },
            api_client=api_client
        )
        self.xpanfileprecreate_endpoint = _Endpoint(
            settings={
                'response_type': (dict,),
                'auth': [],
                'endpoint_path': '/rest/2.0/xpan/file?method=precreate&openapi=xpansdk',
                'operation_id': 'xpanfileprecreate',
                'http_method': 'POST',
                'servers': [
                    {
                        'url': "https://pan.baidu.com",
                        'description': "No description provided",
                    },
                ]
            },
            params_map={
                'all': [
                    'access_token',
                    'path',
                    'isdir',
                    'size',
                    'autoinit',
                    'block_list',
                    'rtype',
                ],
                'required': [
                    'access_token',
                    'path',
                    'isdir',
                    'size',
                    'autoinit',
                    'block_list',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'access_token':
                        (str,),
                    'path':
                        (str,),
                    'isdir':
                        (int,),
                    'size':
                        (int,),
                    'autoinit':
                        (int,),
                    'block_list':
                        (str,),
                    'rtype':
                        (int,),
                },
                'attribute_map': {
                    'access_token': 'access_token',
                    'path': 'path',
                    'isdir': 'isdir',
                    'size': 'size',
                    'autoinit': 'autoinit',
                    'block_list': 'block_list',
                    'rtype': 'rtype',
                },
                'location_map': {
                    'access_token': 'query',
                    'path': 'form',
                    'isdir': 'form',
                    'size': 'form',
                    'autoinit': 'form',
                    'block_list': 'form',
                    'rtype': 'form',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json; charset=UTF-8'
                ],
                'content_type': [
                    'application/x-www-form-urlencoded'
                ]
            },
            api_client=api_client
        )

    def pcssuperfile2(
        self,
        access_token,
        partseq,
        path,
        uploadid,
        type,
        **kwargs
    ):
        """pcssuperfile2  # noqa: E501

        分片上传，这里是实际的文件内容传送部分。一般多为大于4MB的文件，需将文件以4MB为单位切分，对切分后得到的n个分片一一调用该接口进行传送，以实现对原文件的传送（当然若不大于4MB，则直接该对文件进行传送即可）。  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.pcssuperfile2(access_token, partseq, path, uploadid, type, async_req=True)
        >>> result = thread.get()

        Args:
            access_token (str):
            partseq (str):
            path (str):
            uploadid (str):
            type (str):

        Keyword Args:
            file (file_type): 要进行传送的本地文件分片. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            str
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['access_token'] = \
            access_token
        kwargs['partseq'] = \
            partseq
        kwargs['path'] = \
            path
        kwargs['uploadid'] = \
            uploadid
        kwargs['type'] = \
            type
        return self.pcssuperfile2_endpoint.call_with_http_info(**kwargs)

    def xpanfilecreate(
        self,
        access_token,
        path,
        isdir,
        size,
        uploadid,
        block_list,
        **kwargs
    ):
        """xpanfilecreate  # noqa: E501

        将多个文件分片合并成一个文件，生成文件基本信息，完成文件的上传最后一步。  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.xpanfilecreate(access_token, path, isdir, size, uploadid, block_list, async_req=True)
        >>> result = thread.get()

        Args:
            access_token (str):
            path (str): 与precreate的path值保持一致
            isdir (int): isdir
            size (int): 与precreate的size值保持一致
            uploadid (str): precreate返回的uploadid
            block_list (str): 与precreate的block_list值保持一致

        Keyword Args:
            rtype (int): rtype. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            Filecreateresponse
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['access_token'] = \
            access_token
        kwargs['path'] = \
            path
        kwargs['isdir'] = \
            isdir
        kwargs['size'] = \
            size
        kwargs['uploadid'] = \
            uploadid
        kwargs['block_list'] = \
            block_list
        return self.xpanfilecreate_endpoint.call_with_http_info(**kwargs)

    def xpanfileprecreate(
        self,
        access_token,
        path,
        isdir,
        size,
        autoinit,
        block_list,
        **kwargs
    ):
        """xpanfileprecreate  # noqa: E501

        文件预上传，用于获取上传任务id，既uploadid  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.xpanfileprecreate(access_token, path, isdir, size, autoinit, block_list, async_req=True)
        >>> result = thread.get()

        Args:
            access_token (str):
            path (str): 对于一般的第三方软件应用，路径以 \\\"/apps/your-app-name/\\\" 开头。对于小度等硬件应用，路径一般 \\\"/来自：小度设备/\\\" 开头。对于定制化配置的硬件应用，根据配置情况进行填写。
            isdir (int): isdir
            size (int): size
            autoinit (int): autoinit
            block_list (str): 由MD5字符串组成的list

        Keyword Args:
            rtype (int): rtype. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            Fileprecreateresponse
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['access_token'] = \
            access_token
        kwargs['path'] = \
            path
        kwargs['isdir'] = \
            isdir
        kwargs['size'] = \
            size
        kwargs['autoinit'] = \
            autoinit
        kwargs['block_list'] = \
            block_list
        return self.xpanfileprecreate_endpoint.call_with_http_info(**kwargs)




from afeng_tools.fastapi_tool.common.po_service.po_service import PoService


class ResourcePoService(PoService):
    """
    使用示例：resource_po_service = ResourcePoService(app_info.db_code, ResourceInfoPo)
    """

    pass



from afeng_tools.fastapi_tool.common.po_service.po_service import PoService


class SitemapPoService(PoService):
    """
    使用示例：sitemap_po_service = SitemapPoService(app_info.db_code, SitemapInfoPo)
    """

    pass



"""
sqlalchemy工具
"""
from typing import Callable

from sqlalchemy import create_engine
from sqlalchemy.orm import Session, sessionmaker


def create_session_maker(database_uri, echo: bool = True, pool_size: int = 50) -> sessionmaker[Session]:
    """
    创建 SessionMaker
    :param database_uri: 数据库连接地址，如： 'mysql://root:123456@127.0.0.1:3306/video_tools_db'
    :param echo: 是否打印sql
    :param pool_size: 连接池大小
    :return: SessionMaker, 用于创建Session，如：SessionMaker = create_SessionMaker(database_uri); session = SessionMaker()
    """
    return sessionmaker(bind=create_engine(database_uri, echo=echo, pool_size=pool_size),
                        autocommit=False,
                        autoflush=False,
                        expire_on_commit=False)


def create_session(database_uri) -> Session:
    """创建数据库Session"""
    return create_session_maker(database_uri)()


def get_session(session_maker: sessionmaker[Session]) -> Session:
    """获取数据库Session"""
    with session_maker.begin() as session:
        return session


def database_copy_data(database_uri1: str, database_uri2: str, execute_business_fun: Callable, echo: bool = False):
    """
    数据库复制数据
    :param database_uri1: 数据库连接1
    :param database_uri2: 数据库连接2
    :param execute_business_fun: 执行复制业务的函数，该函数需要有两个参数：session1,session2
    :param echo: 是否打印sql
    :return:
    """
    session1 = create_session_maker(database_uri1, echo=echo)()
    session2 = create_session_maker(database_uri2, echo=echo)()
    try:
        execute_business_fun(session1, session2)
    finally:
        session1.close()
        session2.close()


"""
导入工具
"""
import importlib
import threading


def import_jinja2():
    try:
        import jinja2
        return jinja2
    except ModuleNotFoundError:
        jinja2 = None
    return jinja2


def import_module_by_name(module_name: str):
    """通过名称导入模块"""
    return importlib.import_module(module_name)


def import_class(class_full_name: str):
    """
    导入类，示例：ErrorService = import_class('demo.class_demo.ErrorService')
                error_service = ErrorService()
    :param class_full_name: 'demo.class_demo.ErrorService'
    :return:
    """
    _module_name, _class_name = class_full_name.rsplit('.', maxsplit=1)
    # 动态导入模块
    tmp_module = importlib.import_module(_module_name)
    # 从模块中获取类
    return getattr(tmp_module, _class_name)


# 创建一个互斥锁对象
import_lock = threading.Lock()


def import_module_thread_safe(module_name):
    # 获取锁对象
    lock = import_lock.acquire()
    try:
        # 导入模块
        module = importlib.import_module(module_name)
    finally:
        # 释放锁对象
        import_lock.release()
    return module



from typing import Any, TypeVar

from afeng_tools.id_code_tool import id_code_tools
from afeng_tools.sqlalchemy_tools.core.sqlalchemy_enums import SortTypeEnum
from afeng_tools.sqlalchemy_tools.crdu import base_crdu
from sqlalchemy.orm import Query


class PoService:
    ModelType = TypeVar('ModelType', bound=Any)

    def __init__(self, db_code: str, model_type: type):
        self.db_code = db_code
        self.model_type = model_type
        self.ModelType = TypeVar('ModelType', bound=model_type)

    def add(self, po: ModelType) -> ModelType:
        return base_crdu.add(po, db_code=self.db_code)

    def add_batch(self, model_list: list[ModelType]) -> list[ModelType]:
        return base_crdu.add_all(model_list, db_code=self.db_code)

    def update(self, po: ModelType, *criterion) -> ModelType:
        return base_crdu.update(po, *criterion, db_code=self.db_code)

    def update_batch(self, model_list: list[ModelType]) -> list[ModelType]:
        return base_crdu.update_batch(model_list, db_code=self.db_code)

    def save(self, po: ModelType, *criterion, auto_code: bool = False) -> ModelType:
        po = base_crdu.save(po, *criterion, db_code=self.db_code)
        if auto_code:
            if po.code is None:
                po.code = id_code_tools.get_code_by_id(po.id)
                po = base_crdu.update(po, db_code=self.db_code)
        return po

    def save_batch(self, model_list: list[ModelType], *criterion, exist_update: bool = True) -> list[ModelType]:
        return base_crdu.save_batch(model_list, *criterion, exist_update=exist_update, db_code=self.db_code)

    def delete(self, *criterion):
        return base_crdu.delete(self.model_type, *criterion, db_code=self.db_code)

    def create_query(self, *criterion) -> Query:
        return base_crdu.create_query(self.model_type, *criterion, db_code=self.db_code)

    def get(self, *criterion) -> ModelType:
        return base_crdu.query_one(self.model_type, *criterion, db_code=self.db_code)

    def query_more(self, *criterion, sort_column: str = 'order_num',
                   sort_type: SortTypeEnum = SortTypeEnum.desc) -> list[ModelType]:
        return base_crdu.query_all(self.model_type, *criterion, sort_column=sort_column, sort_type=sort_type,
                                   db_code=self.db_code)

    def query_page(self, page_num: int, page_size: int, *criterion,
                   sort_column: str = 'order_num', sort_type: SortTypeEnum = SortTypeEnum.desc) -> list[ModelType]:
        return base_crdu.query_page(self.model_type, page_num, page_size, *criterion,
                                    sort_column=sort_column, sort_type=sort_type,
                                    db_code=self.db_code)

    def count(self, *criterion) -> int:
        return base_crdu.count(self.model_type, *criterion, db_code=self.db_code)



"""
Appium按、点工具类
使用示例：
touch_tool = TouchTool(driver=driver)
touch_tool.press_by_point(x=100, y=300).move_to(x=100, y=100).release()
"""
from appium.webdriver import WebElement
from appium.webdriver.common.touch_action import TouchAction
from appium.webdriver.webdriver import WebDriver


class TouchTool:
    def __init__(self, driver: WebDriver):
        self.driver = driver
        self.touch_action = TouchAction(self.driver)

    def tap(self, el: WebElement, count: int = 1) -> 'TouchTool':
        """轻点"""
        self.touch_action.tap(el, count=count)
        return self

    def tap_by_point(self, x: int, y: int, count: int = 1) -> 'TouchTool':
        """轻点"""
        self.touch_action.tap(x=x, y=y, count=count)
        return self

    def press(self, el: WebElement) -> 'TouchTool':
        """按压"""
        self.touch_action.press(el)
        return self

    def press_by_point(self, x: int, y: int) -> 'TouchTool':
        """按压点"""
        self.touch_action.press(x=x, y=y)
        return self

    def long_press(self, el: WebElement, duration: int = 1000) -> 'TouchTool':
        """长按"""
        self.touch_action.long_press(el, duration=duration)
        return self

    def long_press_by_point(self, x: int, y: int, duration: int = 1000) -> 'TouchTool':
        """长按点"""
        self.touch_action.long_press(x=x, y=y, duration=duration)
        return self

    def wait(self, milli_seconds: int = 1000) -> 'TouchTool':
        """等待（毫秒）"""
        self.touch_action.wait(milli_seconds)
        return self

    def move_to(self, x: int, y: int) -> 'TouchTool':
        """移动"""
        self.touch_action.move_to(x=x, y=y)
        return self

    def release(self):
        """释放按压、长按、移动后的执行操作"""
        self.touch_action.release().perform()
        return self




import re

from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker, Session

from afeng_tools.sqlalchemy_tool.core.sqlalchemy_item import DatabaseInfoItem

# 创建一个Base类
Base = declarative_base()


class SqlalchemyDbTool:
    """db工具类"""
    is_postgresql = True

    def __init__(self, database_info: DatabaseInfoItem):
        self.database_info = database_info
        self.engine = None
        self.session_maker = None
        self._create_engine()
        self._create_session_maker()
        self.is_postgresql = self.get_is_postgresql()

    def _create_engine(self):
        """创建engine"""
        self.engine = create_engine(self.database_info.database_uri,
                                    echo=self.database_info.echo_sql,
                                    pool_size=self.database_info.pool_size,
                                    pool_pre_ping=True, pool_recycle=3600, pool_use_lifo=True)

    def _create_session_maker(self):
        """Session创建者"""
        self.session_maker = sessionmaker(bind=self.engine,
                                          autocommit=self.database_info.auto_commit,
                                          autoflush=self.database_info.auto_flush,
                                          expire_on_commit=self.database_info.expire_on_commit)

    def get_session(self) -> Session:
        """获取Session"""
        return self.session_maker()

    def get_is_postgresql(self) -> bool:
        """是否是postgresql"""
        return True if re.match('^postgresql', self.database_info.database_uri) else False



"""
Random随机数工具
"""
import random
import string
import time


def random_str(length: int = 5, with_timestamp: bool = True):
    """
    返回由【时间戳-length位随机字符】组成的字符串
    :param length: 随机字符长度
    :param with_timestamp: 附加时间戳
    :return: 【时间戳-length位随字符】组成的字符串
    """
    tmp_random_str = ''.join(random.choices(string.ascii_letters + string.digits, k=length))
    if with_timestamp:
        return str(time.time()) + '-' + tmp_random_str
    return tmp_random_str


def random_number_str(length: int = 6):
    """获取随机数字字符串"""
    return ''.join(random.choices(string.digits, k=length))


def random_state(length: int = 8):
    """获取随机生成的state"""
    return ''.join(random.choices(string.ascii_letters + string.digits, k=length)).lower()



"""
文件下载工具
"""
import math
import os.path
import tempfile
from typing import Generator

import requests

from afeng_tools.baidu_pan_tool.core.baidu_pan_models import FileMetaInfo
from afeng_tools.baidu_pan_tool.tools.file_meta_tools import get_file_metas
from afeng_tools.file_tool import file_tools
from afeng_tools.log_tool.loguru_tools import log_error, get_logger

logger = get_logger()


def _get_split_range_list(file_size: int, block_size: int) -> list[str]:
    """
    拆分列表
    :param file_size: 文件大小
    :param block_size: 每块文件的大小，单位B
    :return: {[start,end]}
    """
    part_num = math.ceil(file_size / block_size)
    part_list = []
    for part_seq in range(0, part_num):
        start_point = part_seq * block_size
        if part_seq < part_num - 1:
            end_point = (part_seq + 1) * block_size - 1
            part_list.append(f'{start_point}-{end_point}')
        else:
            part_list.append(f'{start_point}-')
    return part_list


def _download_bytes(access_token: str, down_link: str, header_range=None) -> bytes:
    """
    下载字节
    :param access_token: Access Token
    :param down_link: DLINK下载地址
    :param header_range: 下载文件指定范围的数据
                如：Range: bytes=0-499表示文件起始的500字节；
                    Range: bytes=500-999表示文件的第二个500字节；
                    Range: bytes=-500 表示文件最后500字节；
                    Range: bytes=500-表示文件500字节以后的范围；
                    Range: bytes=0-0,-1表示文件第一个字节和做后一个字节；
                    Range: bytes=500-600,601-999 表示同时指定多个范围。
    :return: bytes
    """
    headers = {
        'User-Agent': 'pan.baidu.com',
        # 'Host': 'd.pcs.baidu.com'
    }
    if header_range:
        headers['Range'] = header_range
    resp = requests.get(down_link + '&access_token=' + access_token, headers=headers, stream=True)
    if resp.status_code < 400:
        return resp.content
    resp.encoding = 'utf-8'
    log_error(logger, f'[BiaduPan]文件下载失败，下载链接没有能下载，响应码是：{resp.status_code}, 响应内容是：{resp.text}')


def _get_downloaded_file(fs_id: int) -> str:
    """获取下载配置文件"""
    tmp_path = tempfile.gettempdir()
    save_path = os.path.join(tmp_path, 'BaiduPan')
    os.makedirs(save_path, exist_ok=True)
    return os.path.join(save_path, f'{fs_id}.download')


def _read_downloaded_range(fs_id: int) -> list[str]:
    """读取已经下载的分片范围列表（当断点续传时），"""
    downloaded_file = _get_downloaded_file(fs_id)
    logger.info(f'[BiaduPan]文件下载，分片信息保存在[{downloaded_file}]')
    if os.path.exists(downloaded_file):
        return file_tools.read_file_lines(downloaded_file)


def _save_downloaded_range(fs_id: int, tmp_range):
    """保存已经下载的分片信息"""
    with open(_get_downloaded_file(fs_id), 'a', encoding='utf-8') as fd:
        fd.write(tmp_range + '\n')


def _delete_downloaded_range_file(fs_id: int):
    """删除下载进度存储文件"""
    os.remove(_get_downloaded_file(fs_id))


def download_bytes(access_token: str, file_meta: FileMetaInfo,
                   max_size: int = 50 * 1000 * 1000,
                   block_size: int = 5 * 1000 * 1000) -> Generator[dict[int, bytes], None, None]:
    """
    下载字节
    :param access_token: Access Token
    :param file_meta: 文件元数据
    :param max_size: 不分片下载的最大大小，默认50M
    :param block_size: 分片的大小, 默认5M
    :return: (开始位置,bytes)
    """
    down_link = file_meta.dlink
    file_size = file_meta.size
    if file_size <= max_size:
        logger.info(f'[BaiduPan]下载文件[{file_meta.path}]')
        yield 0, _download_bytes(access_token=access_token, down_link=down_link)
    else:
        # 分片下载
        split_range_list = _get_split_range_list(file_size, block_size)
        # 读取已经下载的分片
        downloaded_range_list = _read_downloaded_range(file_meta.fs_id)
        for tmp_range in split_range_list:
            if downloaded_range_list and tmp_range in downloaded_range_list:
                continue
            logger.info(f'[BaiduPan]下载文件[{file_meta.path}]-[{tmp_range}]')
            yield int(tmp_range.split('-')[0]), _download_bytes(access_token=access_token, down_link=down_link,
                                                                header_range=f'bytes={tmp_range}')
            _save_downloaded_range(file_meta.fs_id, tmp_range)
        # 删除分片记录文件
        _delete_downloaded_range_file(file_meta.fs_id)


def download_file(access_token: str, fs_id: int, save_path: str,
                  save_file_name: str = None,
                  max_size=5 * 1000 * 1000, block_size=1 * 1000 * 1000) -> str:
    """
    下载文件
    :param access_token: Access Token
    :param fs_id: 文件在云端的唯一标识ID
    :param save_path: 文件的保存路径
    :param save_file_name: 保存的文件名，如果为None，则使用原有文件名
    :param max_size: 不分片下载的最大大小，默认5M
    :param block_size: 分片的大小, 默认1M
    :return: 文件绝对路径
    """
    file_meta_list = get_file_metas(access_token=access_token, fs_id_list=[fs_id], d_link=1)
    if file_meta_list:
        file_meta = file_meta_list[0]
        download_generator = download_bytes(access_token=access_token, file_meta=file_meta,
                                            max_size=max_size, block_size=block_size)
        save_file = os.path.join(save_path, save_file_name if save_file_name else file_meta.filename)
        for start, content in download_generator:
            with open(save_file, 'ab') as f:
                f.seek(start)
                f.write(content)
        logger.info(f'[BaiduPan][{fs_id}]-[{file_meta.path}]下载存储完成，存储文件为:{save_file}')
        return save_file


def get_download_url(access_token: str, fs_id: int) -> str:
    file_meta_list = get_file_metas(access_token=access_token, fs_id_list=[fs_id], d_link=1)
    if file_meta_list:
        file_meta = file_meta_list[0]
        return f'{file_meta.dlink}&access_token={access_token}'



"""
- pip install pydantic -i https://pypi.tuna.tsinghua.edu.cn/simple/
"""
from enum import Enum
from typing import Optional, Any, Union

from pydantic import BaseModel


class SettingsStaticItem(BaseModel):
    """配置静态配置项"""
    # 键名
    name: Optional[str] = None
    # 访问静态资源的url前缀
    url: str
    # 静态文件的所在目录
    path: str


class SettingsKeyItem(BaseModel):
    """配置键项"""
    # 键名
    name: str
    # 值类型(默认是字符串)
    value_type: Union[type, Union[Any]] = str
    # 默认值
    default: Optional[Any] = None
    # 描述
    comment: Optional[str | list] = None
    # 是否必须有
    is_required: Optional[bool] = True


class SettingsKeyEnum(Enum):
    """应用配置枚举"""
    app_name = SettingsKeyItem(name='app.name', comment="应用名", is_required=False)
    app_sub_title = SettingsKeyItem(name='app.sub_title', comment="副标题", is_required=False)
    app_code = SettingsKeyItem(name='app.code', comment="应用编码", is_required=False)
    app_description = SettingsKeyItem(name='app.description', comment="应用描述", is_required=False)
    app_keywords = SettingsKeyItem(name='app.keywords', comment="应用关键字", is_required=False)
    app_db_url = SettingsKeyItem(name='app.db_url', comment="应用数据库",  value_type=Union[str, dict])
    app_active = SettingsKeyItem(name='app.active', comment="激活使用的配置文件，多个之间使用逗号分割",
                                 value_type=Union[str, list[str]])
    app_is_debug = SettingsKeyItem(name='app.is_debug', comment="是否是调试模式", value_type=bool)
    app_is_web = SettingsKeyItem(name='app.is_web', comment="是否是web模式", value_type=bool, default=True)

    # web服务配置
    server_host = SettingsKeyItem(name='server.host', comment="服务器监听地址", value_type=str, default="127.0.0.1",
                                  is_required=False)
    server_port = SettingsKeyItem(name='server.port', comment="服务器监听端口", value_type=int, default=8081,
                                  is_required=False)
    server_origin = SettingsKeyItem(name='server.origin', comment="服务器域信息", value_type=str, is_required=False)
    server_static = SettingsKeyItem(name='server.static', comment="访问静态资源",
                                    value_type=Union[SettingsStaticItem, list[SettingsStaticItem]])
    server_template_path = SettingsKeyItem(name='server.template_path', comment="模板文件路径")

    server_static_save_path = SettingsKeyItem(name='server.static_save_path', comment="静态资源文件路径", value_type=str)

    server_icp_record_info = SettingsKeyItem(name='server.icp_record_info', comment="ICP备案信息")
    server_ga_record_info = SettingsKeyItem(name='server.ga_record_info', comment="公安备案信息")
    server_contact_info = SettingsKeyItem(name='server.contact_info', comment="联系信息")
    server_contact_qq = SettingsKeyItem(name='server.contact_qq', comment="联系QQ")
    server_contact_email = SettingsKeyItem(name='server.contact_email', comment="联系邮箱")

    # 日志配置
    log_info_file = SettingsKeyItem(name='log.info.file', comment="info日志文件", value_type=str,
                                    default='{tmp}/log/{app_code}-{date}-info.log')
    log_info_rotation = SettingsKeyItem(name='log.info.rotation', comment="info日志滚动", value_type=str,
                                        default='50 MB')
    log_error_file = SettingsKeyItem(name='log.error.file', comment="error日志文件", value_type=str,
                                     default='{tmp}/log/{app_code}-{date}-error.log')
    log_error_rotation = SettingsKeyItem(name='log.error.rotation', comment="error日志滚动", value_type=str,
                                         default='50 MB')
    email_login = SettingsKeyItem(name='email.login', comment="登录邮箱", value_type=str)
    email_nickname = SettingsKeyItem(name='email.nickname', comment="登录昵称", value_type=str)
    email_password = SettingsKeyItem(name='email.password', comment="登录密码", value_type=str)

    weixin_app_id = SettingsKeyItem(name='weixin.app_id', comment="微信app_id", value_type=str)
    weixin_app_secret = SettingsKeyItem(name='weixin.app_secret', comment="微信app_secret", value_type=str)
    weixin_token = SettingsKeyItem(name='weixin.token', comment="微信token", value_type=str)
    weixin_encoding_aes_key = SettingsKeyItem(name='weixin.encoding_aes_key', comment="微信encoding_aes_key", value_type=str)


from typing import Any

from afeng_tools.fastapi_tool.common.po_service.po_service import PoService


class LinkPoService(PoService):
    """
    使用示例：link_po_service = LinkPoService(app_info.db_code, LinkInfoPo)
    """

    def query_by_category_code(self, category_code: int) -> list[Any]:
        return self.query_more(self.model_type.category_code == category_code)

    def query_in_category_code(self, category_code_list: list[int]) -> list[Any]:
        return self.query_more(self.model_type.category_code.in_(category_code_list))




import os.path
import time
from typing import Optional

from afeng_tools.application_tool import settings_tools
from afeng_tools.application_tool.settings_enum import SettingsKeyEnum
from afeng_tools.baidu_pan_tool import baidu_pan_tools
from afeng_tools.fastapi_tool import fastapi_router_tools, fastapi_response_tools
from afeng_tools.fastapi_tool.common.enum import ResourceFormatEnum
from afeng_tools.fastapi_tool.common.service.resource_service import ResourceService
from afeng_tools.fastapi_tool.fastapi_response_tools import resp_file
from afeng_tools.log_tool.loguru_tools import get_logger
from fastapi import BackgroundTasks, APIRouter
from starlette.requests import Request
from starlette.responses import RedirectResponse

logger = get_logger()


class ResourceView:
    """
    使用示例：router = ResourceView(app_info.db_code, ResourceInfoPo).router
    """
    def __init__(self, db_code: str, po_model_type: type):
        self.db_code = db_code
        self.po_model_type = po_model_type
        self._router = fastapi_router_tools.create_router(prefix='/resource', tags=['资源'])
        self.resource_service = ResourceService(self.db_code, self.po_model_type)
        self._router.get('/public/{resource_code}', response_class=RedirectResponse)(self.get_public_resource)
        self._router.get('/access/{resource_code}')(self.get_access_resource)
        self._router.get('/download/{resource_code}')(self.get_download_resource)

    @property
    def router(self):
        return self._router

    async def get_public_resource(self, request: Request, background_tasks: BackgroundTasks,
                                  resource_code: Optional[str] = None):
        if resource_code is None or resource_code == 'None' or not resource_code:
            return fastapi_response_tools.resp_404('资源不存在！', request=request)
        if os.path.exists(os.path.join(settings_tools.get_config(SettingsKeyEnum.server_static_save_path), 'resource',
                                       resource_code)):
            return RedirectResponse(f'/static/resource/{resource_code}')
        resource_code = int(resource_code)
        resource_info_po = self.resource_service.get_by_code(resource_code)
        if resource_info_po is None:
            return fastapi_response_tools.resp_404('资源不存在！', request=request)
        if resource_info_po.expire_timestamp and int(time.time()) >= resource_info_po.expire_timestamp:
            if resource_info_po.resource_format == ResourceFormatEnum.image:
                resource_info_po = self.resource_service.refresh_baidu_img_access_url(resource_info_po)
            else:
                return fastapi_response_tools.resp_404('资源不存在！', request=request)
        if resource_info_po.access_url:
            background_tasks.add_task(self.resource_service.run_local_cache, resource_code, resource_info_po.access_url)
            return resource_info_po.access_url
        elif resource_info_po.resource_format == ResourceFormatEnum.image and resource_info_po.baidu_fs_id:
            resource_info_po = self.resource_service.refresh_baidu_img_access_url(resource_info_po)
            if resource_info_po.access_url:
                background_tasks.add_task(self.resource_service.run_local_cache, resource_code,
                                          resource_info_po.access_url)
                return resource_info_po.access_url
        if resource_info_po.local_path:
            return f'/resource/access/{resource_code}'
        return fastapi_response_tools.resp_404('资源不存在！', request=request)

    async def get_access_resource(self, request: Request, resource_code: int):
        resource_info_po = self.resource_service.get_by_code(resource_code)
        if resource_info_po is None:
            return fastapi_response_tools.resp_404('资源不存在！', request=request)
        if resource_info_po.local_path and os.path.exists(resource_info_po.local_path):
            return resp_file(resource_info_po.local_path, resource_info_po.resource_name,
                             resource_info_po.download_flag)
        else:
            return fastapi_response_tools.resp_404('资源不存在！', request=request)

    async def get_download_resource(self, request: Request, resource_code: int):
        resource_info_po = self.resource_service.get_by_code(resource_code)
        if resource_info_po is None:
            return fastapi_response_tools.resp_404('资源不存在！', request=request)
        if resource_info_po.local_path and os.path.exists(resource_info_po.local_path):
            return resp_file(resource_info_po.local_path, resource_info_po.resource_name,
                             resource_info_po.download_flag)
        if resource_info_po.baidu_fs_id:
            download_url = baidu_pan_tools.get_baidu_download_url(resource_info_po)
            if download_url:
                if download_url.startswith('http://'):
                    download_url = 'https://' + download_url.removeprefix('http://')
                return RedirectResponse(download_url, headers={
                    'Host': 'd.pcs.baidu.com',
                    'User-Agent': 'pan.baidu.com'
                })
        return fastapi_response_tools.resp_404('资源不存在！', request=request)



"""
window下图片工具类
- pillow: pip install pillow  -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
"""
from PIL import Image


def show_image(image_file: str, title: str = None):
    """
    显示图片
    :param image_file: 图片文件
    :param title: 标题
    :return: pil_image
    """
    pil_image = Image.open(image_file)
    pil_image.show(title=title)
    return pil_image





"""
通用模型
"""
from afeng_tools.sqlalchemy_tools.core import sqlalchemy_base_model
from sqlalchemy import BigInteger, Column, Text, String, Enum, Boolean, DateTime, func, Float

from afeng_tools.fastapi_tool.common.enum import IconTypeEnum, ResourceFormatEnum, SitemapFreqEnum


def create_group_model(Model: type) -> type:
    class GroupInfoPo(Model):
        """分组信息"""
        __tablename__ = "tb_group_info"
        type_code = Column(String(100), comment='类型类型', default='default', unique=False, nullable=True)
        code = Column(BigInteger, comment='编码', unique=True, nullable=True, index=True)
        title = Column(String(255), comment='标题', unique=False, nullable=True)
        description = Column(Text, comment='描述', unique=False, nullable=True)
        icon_type = Column(Enum(IconTypeEnum, values_callable=lambda x: [i.value.value for i in x]), comment='图标类型',
                           unique=False, nullable=True)
        icon_value = Column(Text, comment='图标值', unique=False, nullable=True)

    return GroupInfoPo


def create_category_model(Model: type) -> type:
    class CategoryInfoPo(Model):
        """分类信息"""
        __tablename__ = "tb_category_info"
        group_code = Column(BigInteger, comment='分组编码', unique=False, nullable=True)
        code = Column(BigInteger, comment='编码', unique=True, nullable=True, index=True)
        title = Column(String(255), comment='标题', unique=False, nullable=True)
        description = Column(Text, comment='描述', unique=False, nullable=True)
        icon_type = Column(Enum(IconTypeEnum, values_callable=lambda x: [i.value.value for i in x]), comment='图标类型',
                           unique=False, nullable=True)
        icon_value = Column(Text, comment='图标值', unique=False, nullable=True)
        parent_code = Column(BigInteger, comment='父编码', unique=False, nullable=True)

    return CategoryInfoPo


def create_resource_model(Model: type) -> type:
    class ResourceInfoPo(Model):
        """资源信息"""
        __tablename__ = "tb_resource_info"
        type_code = Column(String(100), comment='类型类型', default='default', unique=False, nullable=True)
        resource_code = Column(BigInteger, comment='资源编码', index=True, unique=True, nullable=True)
        resource_name = Column(String(255), comment='资源名称', unique=False, nullable=True)
        resource_format = Column(Enum(ResourceFormatEnum), comment='资源格式（image:图片）', unique=False, nullable=True)
        local_path = Column(String(500), comment='本地路径', unique=False, nullable=True)
        baidu_fs_id = Column(BigInteger, comment='百度fs_id', unique=False, nullable=True)
        access_url = Column(String(500), comment='访问url', unique=False, nullable=True)
        expire_timestamp = Column(BigInteger, comment='过期时间戳', unique=False, nullable=True)
        download_flag = Column(Boolean, comment='下载标志', default=False)

    return ResourceInfoPo


def create_link_model(Model: type) -> type:
    class LinkInfoPo(Model):
        """链接信息"""
        __tablename__ = "tb_link_info"
        category_code = Column(BigInteger, comment='分类编码', unique=False, nullable=True)
        code = Column(BigInteger, comment='编码', unique=True, nullable=True, index=True)
        title = Column(String(100), comment='标题', unique=False, nullable=True)
        description = Column(Text, comment='描述', unique=False, nullable=True)
        icon_type = Column(Enum(IconTypeEnum, values_callable=lambda x: [i.value.value for i in x]), comment='图标类型',
                           unique=False, nullable=True)
        icon_value = Column(Text, comment='图标值', unique=False, nullable=True)
        link_href = Column(String(500), comment='链接地址', unique=False, nullable=True)

    return LinkInfoPo


def create_tag_model(Model: type) -> type:
    class TagInfoPo(Model):
        """标签信息"""
        __tablename__ = "tb_tag_info"
        type_code = Column(String(100), comment='类型类型', default='default', unique=False, nullable=True)
        code = Column(BigInteger, comment='编码', unique=True, nullable=True, index=True)
        title = Column(String(100), comment='标题', unique=False, nullable=True)
        description = Column(String(255), comment='描述', unique=False, nullable=True)

    return TagInfoPo


def create_blacklist_model(Model: type) -> type:
    class BlacklistInfoPo(Model):
        """黑名单信息"""
        __tablename__ = "tb_blacklist_info"
        type_code = Column(String(100), comment='类型类型', default='default', unique=False, nullable=True)
        type_value = Column(String(255), comment='类型值', unique=False, nullable=True)

    return BlacklistInfoPo


def create_sitemap_model(Model: type) -> type:
    class SitemapInfoPo(Model):
        """站点地图信息"""
        __tablename__ = "tb_sitemap_info"
        type_code = Column(String(100), comment='类型编码', default='default', unique=False, nullable=True)
        title = Column(String(255), comment='标题', unique=False, nullable=True)
        loc_url = Column(String(500), comment='url地址', unique=False, nullable=True)
        # 0.0 ~ 1.0
        priority = Column(Float, comment='优先权', unique=False, nullable=True)
        last_mod = Column(String(100), comment='最新一次更新时间(YYYY-MM-DD或，YYYY-MM-DDThh:mm:ss)', unique=False,
                          nullable=True)
        change_freq = Column(Enum(SitemapFreqEnum), comment='更新频率',
                             default=SitemapFreqEnum.daily, unique=False, nullable=True)
        parent_id = Column(BigInteger, comment='主键', unique=False, nullable=True)

    return SitemapInfoPo


def create_article_model(Model: type) -> type:
    class ArticleInfoPo(Model):
        """帮助信息"""
        __tablename__ = "tb_article_info"
        type_code = Column(String(100), comment='类型编码', default='default', unique=False, nullable=True)
        code = Column(String(50), comment='编码', unique=False, nullable=True, index=True)
        title = Column(String(255), comment='标题', unique=False, nullable=True)
        sub_title = Column(String(255), comment='副标题', unique=False, nullable=True)
        description = Column(String(255), comment='描述', unique=False, nullable=True)
        icon_type = Column(Enum(IconTypeEnum, values_callable=lambda x: [i.value.value for i in x]), comment='图标类型',
                           unique=False, nullable=True)
        icon_value = Column(Text, comment='图标值', unique=False, nullable=True)
        source_info = Column(String(255), comment='来源信息', unique=False, nullable=True)
        author_info = Column(String(255), comment='作者信息', unique=False, nullable=True)
        publish_time = Column(DateTime, comment='发布时间', default=func.now())
        content = Column(Text, comment='内容', unique=False, nullable=True)

    return ArticleInfoPo


def create_common_models(db_code: str,
                         include_group: bool = True,
                         include_category: bool = True,
                         include_resource: bool = True,
                         include_link: bool = True,
                         include_tag: bool = True,
                         include_blacklist: bool = True,
                         include_sitemap: bool = True,
                         include_article: bool = True) -> tuple[type, ...]:
    """
    创建通用models, 示例： GroupInfoPo, CategoryInfoPo, ResourceInfoPo, LinkInfoPo, TagInfoPo, BlacklistInfoPo, SitemapInfoPo, HelpInfoPo = create_common_models(db_code=app_info.db_code)
    :param db_code:
    :param include_group:
    :param include_category:
    :param include_resource:
    :param include_link:
    :param include_tag:
    :param include_blacklist:
    :param include_sitemap:
    :param include_article:
    :return:
    """
    Model = sqlalchemy_base_model.get_base_model(db_code=db_code)
    type_list = []
    if include_group:
        type_list.append(create_group_model(Model=Model))
    if include_category:
        type_list.append(create_category_model(Model=Model))
    if include_resource:
        type_list.append(create_resource_model(Model=Model))
    if include_link:
        type_list.append(create_link_model(Model=Model))
    if include_tag:
        type_list.append(create_tag_model(Model=Model))
    if include_blacklist:
        type_list.append(create_blacklist_model(Model=Model))
    if include_sitemap:
        type_list.append(create_sitemap_model(Model=Model))
    if include_article:
        type_list.append(create_article_model(Model=Model))

    return tuple(type_list)



from afeng_tools.fastapi_tool.common.po_service.po_service import PoService


class TagPoService(PoService):
    """
    使用示例：tag_po_service = TagPoService(app_info.db_code, TagInfoPo)
    """

    pass
